package com.baijiayun.bjhl_liveplayer;

import static com.baijiayun.bjhl_liveplayer.utils.Utils.checkCameraPermission;
import static com.baijiayun.bjhl_liveplayer.utils.Utils.checkMicPermission;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import com.baijiayun.bjhl_liveplayer.activity.AnnouncementActivity;
import com.baijiayun.bjhl_liveplayer.activity.ChatActivity;
import com.baijiayun.bjhl_liveplayer.activity.OnlineUserActivity;
import com.baijiayun.bjhl_liveplayer.activity.PPTActivity;
import com.baijiayun.bjhl_liveplayer.activity.PullStreamActivity;
import com.baijiayun.bjhl_liveplayer.activity.QuizActivity;
import com.baijiayun.bjhl_liveplayer.activity.RecorderActivity;
import com.baijiayun.bjhl_liveplayer.activity.toolbox.AnswerActivity;
import com.baijiayun.bjhl_liveplayer.activity.toolbox.QaActivity;
import com.baijiayun.bjhl_liveplayer.activity.toolbox.TimerActivity;
import com.baijiayun.bjhl_liveplayer.utils.Utils;
import com.baijiayun.bjyutils.toast.ToastCompat;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.context.LPError;
import com.baijiayun.livebase.models.LPJoinCodeEnterRoomModel;
import com.baijiayun.livecore.LiveSDK;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.listener.LPRoomStatusListener;
import com.baijiayun.livecore.models.imodels.IAnnouncementModel;
import com.baijiayun.livecore.models.roomresponse.LPResCloudRecordModel;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;

public class MainActivity extends AppCompatActivity {
    private LiveRoom liveRoom;
    private CompositeDisposable compositeDisposable = new CompositeDisposable();

    private TextView classNameTv;
    private TextView classStateTv;
    private TextView cloudRecordStatus;

    private ProgressDialog progressDialog;
    private AlertDialog.Builder alertDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        initView();
        Bundle bundle = getIntent().getExtras();
        LiveSDK.customEnvironmentPrefix = bundle.getString(App.CUSTOM_DOMAIN);
        enter(bundle.getString(App.JOIN_CODE), bundle.getString(App.USER_NAME), LPConstants.LPUserType.from(bundle.getInt(App.USER_TYPE)));
    }

    private void initView() {
        classNameTv = findViewById(R.id.main_menu_title);
        classStateTv = findViewById(R.id.main_menu_class_state);
        cloudRecordStatus = findViewById(R.id.cloud_record_status);
    }

    @SuppressLint("NonConstantResourceId")
    public void onClick(View view) {
        if (liveRoom == null) {
            showToast("请先进教室！");
            return;
        }
        switch (view.getId()) {
            case R.id.main_menu_chat: {
                moveToActivityAfterStart(ChatActivity.class);
                break;
            }
            case R.id.main_menu_ppt: {
                moveToActivityAfterStart(PPTActivity.class);
                break;
            }
            case R.id.room_close: {
                if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
                    liveRoom.requestClassEnd();
                } else {
                    showToast("非老师不能开始下课");
                }
                break;
            }
            case R.id.main_menu_recorder: {
                moveToActivityAfterStart(RecorderActivity.class);
                break;
            }
            case R.id.main_menu_quiz: {
                moveToActivityAfterStart(QuizActivity.class);
                break;
            }
            case R.id.main_menu_user: {
                moveToActivity(OnlineUserActivity.class);
                break;
            }
            case R.id.main_menu_remote_video: {
                moveToActivityAfterStart(PullStreamActivity.class);
                break;
            }
            case R.id.announcement_settings: {
                if (liveRoom.isClassStarted()) {
                    if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher
                            || liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                        moveToActivity(AnnouncementActivity.class);
                    } else {
                        showToast("学生获取公告");
                        liveRoom.requestAnnouncement();
                        Button button = (Button) view;
                        button.setTextColor(Color.GRAY);
                        button.setClickable(false);
                    }
                } else {
                    showToast("未上课，不能设置公告");
                }
                break;
            }
            default:
                break;
        }
    }

    public void onClickToolBox(View view) {
        switch (view.getId()) {
            case R.id.toolbox_qa: {
                moveToActivityAfterStart(QaActivity.class);
                break;
            }
            case R.id.toolbox_answer: {
                moveToActivityAfterStart(AnswerActivity.class);
                break;
            }
            case R.id.toolbox_timer: {
                moveToActivityAfterStart(TimerActivity.class);
                break;
            }
            default:
                break;
        }
    }

    public void onClickCloud(View view) {
        switch (view.getId()) {
            case R.id.cloud_record_start:
                liveRoom.requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
                break;

            case R.id.cloud_record_pause:
                liveRoom.requestCloudRecord(LPConstants.CloudRecordStatus.Paused);
                break;

            case R.id.cloud_record_stop:
                liveRoom.requestCloudRecord(LPConstants.CloudRecordStatus.Stopped);
                break;

            default:
                break;
        }
    }

    private void showToast(String msg) {
        ToastCompat.showToast(this, msg, Toast.LENGTH_SHORT);
    }

    private void enter(String code, String name, LPConstants.LPUserType lpUserType) {
        enterRoom(code, name, lpUserType, null);
    }

    private void enterRoom(final String code, final String name, final LPConstants.LPUserType lpUserType, final String userAvatar) {
        progressDialog = null;
        LPJoinCodeEnterRoomModel lpJoinCodeEnterRoomModel = new LPJoinCodeEnterRoomModel(code, name, userAvatar, lpUserType);
        liveRoom = LiveSDK.enterRoom(getApplicationContext(), lpJoinCodeEnterRoomModel, new LPRoomStatusListener() {
            @Override
            public void onLaunchSteps(int step, int totalStep) {
                Log.i("init steps", "step:" + step + "/" + totalStep);
                // 设置简易进度条
                if (progressDialog == null) {
                    progressDialog = new ProgressDialog(MainActivity.this);
                    progressDialog.setTitle("进入直播间中");
                    progressDialog.setCancelable(false);
                    progressDialog.setMax(4);
                    progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
                    progressDialog.show();
                }
                progressDialog.setProgress(step);
            }

            @Override
            public void onLaunchError(LPError error) {
                Log.e("error", error.getCode() + " " + error.getMessage());
                Toast.makeText(MainActivity.this, error.getMessage() + error.getCode(), Toast.LENGTH_SHORT).show();
                finish();
            }

            @Override
            public void onLaunchSuccess(LiveRoom liveRoom) {
                progressDialog.dismiss();
                onInitSuccess(liveRoom);
            }

            @Override
            public void onLivingError(LPError error) {

            }

            @Override
            public void onQuitRoom() {

            }
        });
        App.liveRoom = liveRoom;
    }

    private void onInitSuccess(LiveRoom liveRoom) {
        checkCameraPermission(this);
        checkMicPermission(this);
        this.liveRoom = liveRoom;
        // 课程名
        classNameTv.setText("课程名：" + liveRoom.getRoomInfo().title);
        if (liveRoom.getCurrentUser().getType() != LPConstants.LPUserType.from(getIntent().getExtras().getInt(App.USER_TYPE))) {
            showToast("账号身份不符");
            finish();
        }
        subscribe();
    }

    private void subscribe() {
        // 上课监听
        compositeDisposable.add(liveRoom.getObservableOfClassStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(a -> {
                    showToast("上课啦！！");
                    classStateTv.setText(getString(R.string.in_class));
                    if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                        progressDialog.dismiss();
                    }
                }));

        // 下课监听
        compositeDisposable.add(liveRoom.getObservableOfClassEnd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(a -> {
                    showToast("下课啦！！");
                    classStateTv.setText(getString(R.string.no_class));
                    if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                        progressDialog = null;
                        progressDialog = new ProgressDialog(this);
                        progressDialog.setTitle("下课啦");
                        progressDialog.setMessage("等待老师上课中。。。");
                        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                        progressDialog.setCancelable(false);
                        progressDialog.setButton(DialogInterface.BUTTON_NEGATIVE, "结束等待，返回登录界面", (dialog, which) -> {
                            progressDialog.dismiss();
                            finish();
                        });
                        progressDialog.show();
                    } else {
                        finish();
                    }
                }));

        //用户进入房间监听
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfUserIn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(a -> {
                    showToast(a.getUser().getName() + "进入房间");
                }));

        //用户退出房间监听
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfUserOut()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(a -> {
                    showToast(a.getName() + "退出房间");
                }));

        //更改公告监听
        compositeDisposable.add(liveRoom.getObservableOfAnnouncementChange()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((IAnnouncementModel iAnnouncementModel) -> {
                    AlertDialog.Builder builder = new AlertDialog.Builder(this);
                    //点击周围可以取消
                    builder.setCancelable(true);
                    builder.setTitle("公告");
                    if (!TextUtils.isEmpty(iAnnouncementModel.getLink())) {
                        builder.setNegativeButton("点击跳转到链接", (dialog, which) -> {
                            showToast("跳转到" + iAnnouncementModel.getLink());
                        });
                    }
                    if (TextUtils.isEmpty(iAnnouncementModel.getContent())) {
                        builder.setMessage("公告没有内容");
                    } else {
                        builder.setMessage(iAnnouncementModel.getContent());
                        builder.setPositiveButton("确认收到", (dialog, which) -> {
                        });
                    }
                    builder.show();
                }));

        //云端录制监听
        compositeDisposable.add(liveRoom.getObservableOfCloudRecordStatus()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((LPResCloudRecordModel lpRecordValueModel) -> {
                    if (lpRecordValueModel.status == 1) {
                        cloudRecordStatus.setText("正在录制");
                    } else if (lpRecordValueModel.status == 2) {
                        cloudRecordStatus.setText("暂停录制");
                    } else if (lpRecordValueModel.status == 0) {
                        cloudRecordStatus.setText("停止录制");
                    }
                }));

        //实现账号的单点登录
        compositeDisposable.add(liveRoom.getObservableOfLoginConflict()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iLoginConflictModel -> {
                    int type = iLoginConflictModel.getConflictEndType().getType();
                    showLoginConflict(type);
                }));

        // 上课状态
        if (liveRoom.isClassStarted()) {
            classStateTv.setText(getString(R.string.in_class));
        } else {
            classStateTv.setText(getString(R.string.no_class));
            if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher ||
                    liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                if (alertDialog == null) {
                    alertDialog = new AlertDialog.Builder(MainActivity.this);
                }
                alertDialog.setTitle("未上课");
                alertDialog.setMessage("当前还未上课，是否开始上课？");
                alertDialog.setCancelable(false);
                alertDialog.setPositiveButton("是", (dialog, which) -> liveRoom.requestClassStart());
                alertDialog.setNegativeButton("否", (dialog, which) -> finish());
                alertDialog.show();
            }
        }
    }

    //显示登陆冲突提示框
    private void showLoginConflict(int type) {

        String conflictClient = Utils.convertTypeToString(type);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        builder.setMessage("您的账号在" + conflictClient + "上登陆");

        builder.setPositiveButton("确认", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                finish();
            }
        });
        builder.create().show();
    }

    public void moveToActivityAfterStart(Class cla) {
        if (liveRoom.isClassStarted()) {
            moveToActivity(cla);
        } else {
            showToast("未上课，不能进行此操作");
        }
    }

    public void moveToActivity(Class cla) {
        Intent intent = new Intent(this, cla);
        startActivity(intent);
    }


    @Override
    protected void onDestroy() {
        compositeDisposable.clear();
        liveRoom.quitRoom();
        App.liveRoom = null;
        super.onDestroy();
    }
}
