package com.baijiayun.bjhl_liveplayer.activity;

import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import com.baijiayun.bjhl_liveplayer.App;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.toast.ToastCompat;
import com.baijiayun.livebase.context.LPError;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.listener.LPRoomStatusListener;

import io.reactivex.disposables.CompositeDisposable;

public abstract class BaseActivity extends AppCompatActivity {
    protected CompositeDisposable compositeDisposable;
    protected LiveRoom liveRoom;
    View mRootView;

    protected void handleLivingError(LPError error) {
        ToastCompat.showToast(BaseActivity.this, error.getMessage(), Toast.LENGTH_LONG);
        LPLogger.e("reconnect");
        switch ((int)error.getCode()) {
            case LPError.CODE_ERROR_ROOMSERVER_LOSE_CONNECTION:
            case LPError.CODE_ERROR_NETWORK_FAILURE:
                new AlertDialog.Builder(BaseActivity.this)
                        .setTitle("网络断开了")
                        .setPositiveButton("继续连接", (dialog, which) -> {
                            reconnect();
                        })
                        .setNegativeButton("退出直播间", (dialog, which) -> finish())
                        .create()
                        .show();

                break;
            case LPError.CODE_ERROR_NETWORK_MOBILE:
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        mRootView = View.inflate(this,getContentViewId(),null);
        setContentView(mRootView);
        liveRoom = App.liveRoom;
        initView();
        compositeDisposable = new CompositeDisposable();
        if (liveRoom != null) {
            subscribe();
        }
    }

    private void reconnect() {
        liveRoom.reconnect(new LPRoomStatusListener() {

            @Override
            public void onLaunchSteps(int step, int totalStep) {

            }

            @Override
            public void onLaunchError(LPError error) {
                Toast.makeText(BaseActivity.this, "重进房间失败 " + error.getMessage(), Toast.LENGTH_LONG).show();
            }

            @Override
            public void onLaunchSuccess(LiveRoom liveRoom) {
                subscribe();
            }

            @Override
            public void onLivingError(LPError error) {
                handleLivingError(error);
            }

            @Override
            public void onQuitRoom() {

            }
        });
    }

    protected abstract int getContentViewId();

    protected abstract void subscribe();

    protected abstract void initView();

    protected void setOnClickListener(int id, View.OnClickListener onClickListener) {
        View view = mRootView.findViewById(id);
        if (view != null) {
            view.setOnClickListener(onClickListener);
        }
    }

    protected void setEnable(int id, boolean enable) {
        View view = mRootView.findViewById(id);
        if (view != null) {
            view.setEnabled(enable);
        }
    }

    public <T extends View> T findViewById(int id) {
        return mRootView.findViewById(id);
    }

    protected void showToast(String msg) {
        ToastCompat.showToast(this, msg, Toast.LENGTH_SHORT);
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        compositeDisposable.dispose();
    }
}
