package com.baijiayun.bjhl_liveplayer.activity;

import android.content.Intent;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.fragment.app.DialogFragment;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiahulian.common.networkv2.BJProgressCallback;
import com.baijiahulian.common.networkv2.BJResponse;
import com.baijiahulian.common.networkv2.HttpException;
import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjhl_liveplayer.adapter.ExpressionAdapter;
import com.baijiayun.bjhl_liveplayer.adapter.MessageAdapter;
import com.baijiayun.bjhl_liveplayer.fragment.OperateMessageFragment;
import com.baijiayun.bjhl_liveplayer.utils.FileUtil;
import com.baijiayun.bjhl_liveplayer.utils.Utils;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPExpressionModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.LPChatMessageParser;
import com.baijiayun.livebase.widgets.emoji.IExpressionModel;
import com.baijiayun.livecore.models.LPRoomForbidChatResult;
import com.baijiayun.livecore.models.imodels.IMessageModel;
import com.baijiayun.livecore.viewmodels.ChatVM;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;


public class ChatActivity extends BaseActivity implements OperateMessageFragment.MessageOperationListener {

    public static final int REQUEST_GET_CONTENT_CODE = 0;
    private MessageAdapter messageAdapter;
    private List<IMessageModel> messageList;
    private Disposable disposeOfNotifyDataChange;
    private ChatVM chatVM;
    private IUserModel toUser;
    private boolean isPrivateChat;

    private HashMap<String, String> expressions;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_chat;
    }

    @Override
    protected void subscribe() {
        //聊天消息监听
        compositeDisposable.add(chatVM.getObservableOfReceiveMessage()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::onUpdateChat));

        cbMuteAll.setTag(true);

        // 全员禁言监听
        compositeDisposable.add(liveRoom.getChatVM().getObservableOfForbidAllChatStatus()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(forbidChatResult -> {
                    if (forbidChatResult.type == LPConstants.LPForbidChatType.TYPE_ALL) {
                        updateMuteView(forbidChatResult.isForbid);
                        cbMuteAll.setChecked(forbidChatResult.isForbid);
                    }
                }));

        // 单人禁言状态
        compositeDisposable.add(liveRoom.getChatVM().getObservableOfForbidChat()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iForbidChatModel -> {
                    showToast(iForbidChatModel.getForbidUser().getName() + "被禁言" + iForbidChatModel.getDuration() + "s");
                }));

        disposeOfNotifyDataChange = chatVM.getObservableOfNotifyDataChange()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(messageModelList -> {
                    initRecyclerView(messageModelList);
                    Log.d("ChatActivity", "getNotifyDataChange");
                    disposeOfNotifyDataChange.dispose();
                });

        compositeDisposable.add(chatVM.getObservableOfMsgRevoke()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpMessageRevoke -> {
                    messageAdapter.notifyDataSetChanged();
                }));

        compositeDisposable.add(chatVM.getObservableOfReceiveTranslateMessage()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpMessageTranslateModel -> {
                    showToast(lpMessageTranslateModel.result);
                }));

        compositeDisposable.add(chatVM.getObservableOfMsgStickyList()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMessageModels -> {
                    if (iMessageModels.size() == 0) {
                        stickMessageContainerRl.setVisibility(View.GONE);
                    }
                    for (IMessageModel messageModel : iMessageModels) {
                        stickMessageContainerRl.setVisibility(View.VISIBLE);
                        stickyMessageTv.setText(messageModel.getContent());
                    }
                }));
    }

    RecyclerView rvMessage;
    RecyclerView rvExpression;
    TextView tvMuteState;
    AppCompatCheckBox cbMuteAll;
    EditText etEdit;
    TextView stickyMessageTv;
    RelativeLayout stickMessageContainerRl;
    TextView toUserNameTv;

    @Override
    protected void initView() {
        etEdit = findViewById(R.id.main_menu_chat_edit);
        cbMuteAll = findViewById(R.id.main_menu_chat_mute_all);
        tvMuteState = findViewById(R.id.main_menu_chat_mute_state);
        rvMessage = findViewById(R.id.message_recyclerview);
        rvExpression = findViewById(R.id.expression_recyclerview);
        stickyMessageTv = findViewById(R.id.sticky_message_textview);
        stickMessageContainerRl = findViewById(R.id.sticky_message_container);
        toUserNameTv = findViewById(R.id.to_user_name_text);
        chatVM = liveRoom.getChatVM();
        initExpressionRecyclerView();

        setOnClickListener(R.id.main_menu_chat_send, view -> {
            String content = etEdit.getEditableText().toString();
            if (TextUtils.isEmpty(content)) {
                return;
            }
            // 发送聊天消息
            if (isPrivateChat) {
                chatVM.sendMessageToUser(toUser, content);
                etEdit.setText("");
                toUserNameTv.setVisibility(View.GONE);
                isPrivateChat = false;
            } else {
                chatVM.sendMessage(content);
                etEdit.setText("");
            }
        });

        setOnClickListener(R.id.cancel_sticky_message, view -> {
            List<IMessageModel> list = new ArrayList<IMessageModel>();
            chatVM.requestMsgStickyList(list);
        });

        cbMuteAll.setTag(false);

        cbMuteAll.setOnCheckedChangeListener((compoundButton, checked) -> {
            if (!(boolean) cbMuteAll.getTag()) {
                return;
            }
            if (liveRoom.isTeacherOrAssistant()) {
                liveRoom.getChatVM().requestForbidAllChat(checked);
            } else {
                showToast("仅老师或助教可禁言");
            }
        });

        setOnClickListener(R.id.select_image, view -> {
            Utils.checkFilePermission(this);
            Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
            intent.setType("image/*");
            startActivityForResult(intent, REQUEST_GET_CONTENT_CODE);
        });

    }

    private void initExpressionRecyclerView() {
        GridLayoutManager manager = new GridLayoutManager(this, 3);
        manager.setOrientation(RecyclerView.HORIZONTAL);
        rvExpression.setLayoutManager(manager);
        List<LPExpressionModel> iExpressionModels = chatVM.getExpressions();
        ExpressionAdapter adapter = new ExpressionAdapter(this, iExpressionModels);
        rvExpression.setAdapter(adapter);
        adapter.setOnExpressionClickListener(expressionModel -> {
            String expressionFormat = String.format("[%s]", expressionModel.getKey());
            String s = etEdit.getText() + expressionFormat;
            etEdit.setText(s);
            etEdit.setSelection(s.length());
        });
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode != RESULT_OK) {
            return;
        }
        if (requestCode == 0) {
            Uri uri = data.getData();
            String realPathFromURI = FileUtil.getPath(this, uri);
            sendImageByImagePath(realPathFromURI);
        }
    }

    private void sendImageByImagePath(String path) {
        liveRoom.getChatVM().getObservableOfUploadImage(path)
                .subscribe(uploadModel -> {
                    String imageContent = LPChatMessageParser.toImageMessage(uploadModel.url);
                    if (isPrivateChat) {
                        chatVM.sendImageMessageToUser(toUser, imageContent, uploadModel.width, uploadModel.height);
                        isPrivateChat = false;
                        runOnUiThread(() -> toUserNameTv.setVisibility(View.GONE));
                    } else {
                        chatVM.sendImageMessage(imageContent, uploadModel.width, uploadModel.height);
                    }
                });
    }

    private void initRecyclerView(List<IMessageModel> messageModelList) {
        expressions = new HashMap<>();
        for (IExpressionModel iExpressionModel: chatVM.getExpressions()) {
            expressions.put("[" + iExpressionModel.getName() + "]", iExpressionModel.getUrl());
            expressions.put("[" + iExpressionModel.getKey() + "]", iExpressionModel.getUrl());
            expressions.put("[" + iExpressionModel.getNameEn() + "]", iExpressionModel.getUrl());
        }
        messageList = new ArrayList<>(messageModelList);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        linearLayoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        rvMessage.setLayoutManager(linearLayoutManager);
        messageAdapter = new MessageAdapter(this, messageList, liveRoom.getCurrentUser());
        messageAdapter.setExpressions(expressions);
        rvMessage.setAdapter(messageAdapter);
        rvMessage.smoothScrollToPosition(messageList.size());

        messageAdapter.setOnItemClickListener(new MessageAdapter.OnItemClickListener() {
            @Override
            public void onItemLongClick(IMessageModel messageModel) {
                OperateMessageFragment operateMessageFragment = new OperateMessageFragment(messageModel);
                operateMessageFragment.show(getSupportFragmentManager(), "operateMessage");
            }
        });
    }


    /**
     * 更新聊天记录视图
     *
     * @param iMessageModel 新的聊天消息
     */
    private void onUpdateChat(IMessageModel iMessageModel) {
        messageAdapter.addData(iMessageModel);
        rvMessage.smoothScrollToPosition(messageList.size());
        Log.d("ChatActivity", "onUpdateChat");
    }

    /**
     * 更新禁言视图
     *
     * @param isMute 是否被禁言
     */
    private void updateMuteView(boolean isMute) {
        setEnable(R.id.main_menu_chat_send, !isMute);
        String message;
        if (isMute) {
            message = getString(R.string.can_not_send_message);
        } else {
            message = getString(R.string.can_send_message);
        }
        tvMuteState.setText(message);
    }

    @Override
    public void revokeMessage(IMessageModel messageModel) {
        if (messageModel.getFrom().getName().equals(liveRoom.getCurrentUser().getName())) {
            chatVM.requestMsgRevoke(messageModel.getId(), messageModel.getFrom().getUserId());
        } else {
            showToast("只能撤回自己发的消息哦！");
        }
    }

    @Override
    public void stickMessage(IMessageModel messageModel) {
        List<IMessageModel> iMessageModels = new ArrayList<IMessageModel>();
        iMessageModels.add(messageModel);
        chatVM.requestMsgStickyList(iMessageModels);
    }

    @Override
    public void translateMessage(IMessageModel messageModel) {
        if (messageModel.getMessageType() == LPConstants.MessageType.Text) {
            chatVM.sendTranslateMessage(messageModel.getContent()
                    , messageModel.getId()
                    , String.valueOf(liveRoom.getRoomInfo().roomId)
                    , liveRoom.getCurrentUser().getUserId()
                    , "zh"
                    , "en"
            );
        } else {
            showToast("只有文本消息才可以翻译哦");
        }
    }

    @Override
    public void sendMessageToUser(IMessageModel messageModel) {
        toUser = messageModel.getFrom();
        toUserNameTv.setText(String.format("私发:%s", toUser.getName()));
        toUserNameTv.setVisibility(View.VISIBLE);
        isPrivateChat = true;
    }
}