package com.baijiayun.bjhl_liveplayer.activity;


import android.os.Build;
import android.util.Log;
import android.widget.TextView;

import androidx.annotation.RequiresApi;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjhl_liveplayer.adapter.UserAdapter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomBlockedUserModel;
import com.scwang.smartrefresh.layout.SmartRefreshLayout;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.android.schedulers.AndroidSchedulers;

public class OnlineUserActivity extends BaseActivity {
    private static final String TAG = "OnlineUserActivity";
    private UserAdapter userAdapter;
    private SmartRefreshLayout smartRefreshLayout;
    private TextView classUserNumberTv;
    private int onlineCount;

    private List<UserAdapter.UserCache> caches;
    private List<UserAdapter.UserCache> blockedCaches;
    private Map<String, Integer> userNumber2Position;

    private UserAdapter userAdapterBlocked;
    private SmartRefreshLayout smartRefreshLayoutBlocked;
    private TextView classUserNumberBlocked;
    private RecyclerView userListRv;
    private RecyclerView userBlockedRv;

    private String toInviteUser;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_online_user;
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    protected void subscribe() {
        // 房间人数变化监听：无法主动触发，用户进出触发
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfOnLineUserCount()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(integer -> {
                    classUserNumberTv.setText("当前在线人数为：" + integer);
                }));

        // 在线用户列表：用户进用户出、loadmore触发
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfOnlineUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((List<IUserModel> iUserModels) -> {
                    caches = initCachesWithStatus(iUserModels);
                    userAdapter.setUserlist(caches);
                }));

        // 黑名单列表：无法主动触发，拉黑和移除拉黑都会触发此回调
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfBlockedUserList()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((List<IUserModel> iUserModels) -> {
                    // liveroom拉黑
                    showToast("得到拉黑列表");
                    blockedCaches = initBlockedList(iUserModels);
                    userAdapterBlocked.setUserlist(blockedCaches);
                    classUserNumberBlocked.setText("当前被拉黑人数：" + iUserModels.size());
                }));

        // 拉黑回调
        compositeDisposable.add(liveRoom.getOnlineUserVM().getObservableOfBlockedUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((LPResRoomBlockedUserModel lpResRoomBlockedUserModel) -> {
                    showToast("拉黑了" + lpResRoomBlockedUserModel.blockedUser.name);
                }));

        // 邀请发言回调
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakInvite()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpSpeakInviteModel -> {
                    toInviteUser = liveRoom.getOnlineUserVM().getUserById(lpSpeakInviteModel.to).getNumber();
                    // 此处position必不为null，因为被邀请发言的user已经存在于caches中，所以理应同步存在于map中
                    Integer position = userNumber2Position.get(toInviteUser);
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache userCache = caches.get(position);
                    UserAdapter.Status.SPEAK_STATUS speakStatus;
                    if (lpSpeakInviteModel.invite == 1) {
                        speakStatus = liveRoom.getCurrentUser().getNumber().equals(lpSpeakInviteModel.to)
                                ? UserAdapter.Status.SPEAK_STATUS.Invited_Me
                                : UserAdapter.Status.SPEAK_STATUS.Invited;
                    } else {
                        speakStatus = liveRoom.getCurrentUser().getNumber().equals(lpSpeakInviteModel.to)
                                ? UserAdapter.Status.SPEAK_STATUS.Invited_Cancel_Me
                                : UserAdapter.Status.SPEAK_STATUS.Invited_Cancel;
                    }
                    userCache.status.speakStatus = speakStatus;
                    caches.set(position,userCache);
                    userAdapter.setUserlist(caches);
                }));

        // 学生是否接收邀请发言的响应
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakInviteRes()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpSpeakInviteConfirmModel -> {
                    // 当toInviteUser为null，说明是要么没邀请就拒绝（接受），要么就是连续收到两次拒绝（接受），以第一次为准
                    // 其实这里可以规避这个问题，就是对接受（拒绝）按钮进行隐藏，但是会涉及到布局的设计问题，这里对功能展示没有影响所以作罢
                    if (toInviteUser != null) {
                        Integer position = userNumber2Position.get(toInviteUser);
                        if (position == null) {
                            Log.e(TAG, "map do not sync", new NullPointerException());
                            return;
                        }
                        UserAdapter.UserCache userCache = caches.get(position);
                        UserAdapter.Status.SPEAK_STATUS speakStatus;
                        if (lpSpeakInviteConfirmModel.confirm == 1) {
                            speakStatus = UserAdapter.Status.SPEAK_STATUS.Invited_Allow;
                        } else {
                            speakStatus = UserAdapter.Status.SPEAK_STATUS.Invited_Refuse;
                        }
                        userCache.status.speakStatus = speakStatus;
                        caches.set(position, userCache);
                        userAdapter.setUserlist(caches);
                    }
                    toInviteUser = null;
                }));

        // 老师监听学生举手
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakApply()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    Integer position = userNumber2Position.get(iMediaModel.getUser().getNumber());
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache cache = caches.get(position);
                    cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.HandsUp;
                    caches.set(position,cache);
                    userAdapter.setUserlist(caches);
                }));

        // 老师监听学生取消举手
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakApplyDeny()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    Integer position = userNumber2Position.get(iMediaModel.getUser().getNumber());
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache cache = caches.get(position);
                    cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.HandsUp_Cancel;
                    caches.set(position,cache);
                    userAdapter.setUserlist(caches);
                }));

        // 学生监听agreeSpeakApply或者disagreeSpeakApply
        // 学生取消举手\主动下麦也会回调这个，可以用来展示学生下麦
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    Integer position = userNumber2Position.get(iMediaControlModel.getUser().getNumber());
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache cache = caches.get(position);
                    if (iMediaControlModel.isApplyAgreed()) {
                        cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking;
                    } else {
                        // sender ?= teacher
                        IUserModel currentUser = liveRoom.getCurrentUser();
                        // liveroom.getTeacherUser 为空
                        if (currentUser.getType() == LPConstants.LPUserType.Teacher || currentUser.getType() == LPConstants.LPUserType.Assistant) {
                            // 当前用户是老师或助教
                            if (iMediaControlModel.getSenderUserId().equals(currentUser.getUserId())) {
                                // 发送取消举手的是老师
                                cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking_Tea_Cancel;
                            } else {
                                cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking_Stu_Cancel;
                            }
                        } else {
                            // 当前用户是学生
                            if (iMediaControlModel.getSenderUserId().equals(currentUser.getUserId())) {
                                // 发送取消举手的是老师
                                cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking_Stu_Cancel;
                            } else {
                                cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking_Tea_Cancel;
                            }
                        }
                    }
                    caches.set(position,cache);
                    userAdapter.setUserlist(caches);
                }));

        // 老师远程控制学生回调
        // 好像由于没有真正进入发言（音视频流）导致老师的closeotherspeak没有被真正触发
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfMediaControl()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    Integer position = userNumber2Position.get(iMediaControlModel.getUser().getNumber());
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache cache = caches.get(position);
                    if (iMediaControlModel.isApplyAgreed()) {
                        cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking;
                    } else {
                        cache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Speaking_Tea_Cancel;
                    }
                    caches.set(position,cache);
                    userAdapter.setUserlist(caches);
                }));

        // 收到画笔权限改变回调，权限未改变不会回调
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getPublishSubjectOfDrawingAuth()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    // TODO: 无论是授予还是收回权限，返回的aBoolean都为false？
                    if (liveRoom.isTeacherOrAssistant()) {
                        Integer position = userNumber2Position.get(userAdapter.pollFirstFromPending());
                        if (position == null) {
                            Log.e(TAG, "map do not sync", new NullPointerException());
                            return;
                        }
                        UserAdapter.UserCache cache = caches.get(position);
                        if (cache.status.drawStatus == UserAdapter.Status.DRAW_STATUS.No) {
                            cache.status.drawStatus = UserAdapter.Status.DRAW_STATUS.Have_DrawingAuth_Tea;
                        } else {
                            cache.status.drawStatus = UserAdapter.Status.DRAW_STATUS.No;
                        }
                        caches.set(position,cache);
                        userAdapter.setUserlist(caches);
                    }
                }));

        // 所有学生都收到画笔权限改变回调
        compositeDisposable.add(liveRoom.getSpeakQueueVM().getPublishSubjectOfStudentDrawingAuth()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    Integer position = userNumber2Position.get(liveRoom.getCurrentUser().getNumber());
                    if (position == null) {
                        Log.e(TAG, "map do not sync", new NullPointerException());
                        return;
                    }
                    UserAdapter.UserCache cache = caches.get(position);
                    if (aBoolean) {
                        cache.status.drawStatus = UserAdapter.Status.DRAW_STATUS.Have_DrawingAuth_Stu;
                    } else {
                        cache.status.drawStatus = UserAdapter.Status.DRAW_STATUS.No;
                    }
                    caches.set(position,cache);
                    userAdapter.setUserlist(caches);
                }));


        liveRoom.getOnlineUserVM().loadMoreUser();
        onlineCount = liveRoom.getOnlineUserVM().getOnlineUserList().size();
        //初始时获取黑名单列表，后面拉黑和移除拉黑都会触发回调
        List<IUserModel> list = liveRoom.getOnlineUserVM().getBlockedUserList();
        blockedCaches = initBlockedList(list);
        userAdapterBlocked.setUserlist(blockedCaches);
        classUserNumberBlocked.setText("当前被拉黑人数：" + list.size());
    }

    @Override
    protected void initView() {
        // 创建number to status的map
        userNumber2Position = new HashMap<>(16);

        // 在线用户
        classUserNumberTv = findViewById(R.id.class_user_number);
        userListRv = findViewById(R.id.activity_userList);
        smartRefreshLayout = findViewById(R.id.activity_userList_layout);
        userAdapter = new UserAdapter(this, liveRoom, false);
        userListRv.setLayoutManager(new LinearLayoutManager(this));
        userListRv.setAdapter(userAdapter);
        smartRefreshLayout.setOnRefreshListener(refreshLayout -> {
            onlineCount = liveRoom.getOnlineUserVM().getOnlineUserList().size();
            classUserNumberTv.setText("当前在线人数为：" + onlineCount);
            liveRoom.getOnlineUserVM().loadMoreUser();
            smartRefreshLayout.finishRefresh(2000);
        });

        // 黑名单用户
        classUserNumberBlocked = findViewById(R.id.class_user_number_blocked);
        userBlockedRv = findViewById(R.id.activity_userList_blocked);
        smartRefreshLayoutBlocked = findViewById(R.id.activity_userList_blocked_layout);
        userAdapterBlocked = new UserAdapter(this, liveRoom, true);
        userBlockedRv.setLayoutManager(new LinearLayoutManager(this));
        userBlockedRv.setAdapter(userAdapterBlocked);
        smartRefreshLayoutBlocked.setOnRefreshListener(refreshLayout -> {
            //请求黑名单List
            List<IUserModel> list = liveRoom.getOnlineUserVM().getBlockedUserList();
            blockedCaches = initBlockedList(list);
            userAdapterBlocked.setUserlist(blockedCaches);
            classUserNumberBlocked.setText("当前被拉黑人数：" + list.size());
            smartRefreshLayoutBlocked.finishRefresh(2000);
        });
    }

    // 初始化onlineuser，list为在线用户列表，因此要跟number2position同步
    private List<UserAdapter.UserCache> initCachesWithStatus(List<IUserModel> list){
        ArrayList<UserAdapter.UserCache> userCaches = new ArrayList<>();
        if (userNumber2Position == null) {
            userNumber2Position = new HashMap<>(16);
        }
        HashMap<String, Integer> hashMap = new HashMap<>(16);
        for (int i = 0; i < list.size(); i++){
            UserAdapter.UserCache userCache = new UserAdapter.UserCache();
            userCache.iUserModel = list.get(i);
            // 同步状态缓存
            Integer position = userNumber2Position.get(userCache.iUserModel.getNumber());
            if (position != null) {
                // 该user已在caches中存在，需要将以前的状态更新到当前，并将当前位置更新到map
                userCache.status = caches.get(position).status;
            }
            // 若user曾不存在，则需要作为新user增加到map中，状态为online
            userNumber2Position.put(userCache.iUserModel.getNumber(), i);
            hashMap.put(userCache.iUserModel.getNumber(), i);
            userCaches.add(userCache);
        }
        userNumber2Position.clear();
        userNumber2Position = hashMap;
        return userCaches;
    }

    private List<UserAdapter.UserCache> initBlockedList(List<IUserModel> list) {
        ArrayList<UserAdapter.UserCache> userCaches = new ArrayList<>();
        for (IUserModel iUserModel : list){
            UserAdapter.UserCache userCache = new UserAdapter.UserCache();
            userCache.iUserModel = iUserModel;
            userCache.status.speakStatus = UserAdapter.Status.SPEAK_STATUS.Blocked;
            userCache.status.drawStatus = UserAdapter.Status.DRAW_STATUS.Blocked;
            userCaches.add(userCache);
        }
        return userCaches;
    }

}