package com.baijiayun.bjhl_liveplayer.activity;

import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.RequiresApi;
import androidx.fragment.app.DialogFragment;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjhl_liveplayer.fragment.SelectColorFragment;
import com.baijiayun.bjhl_liveplayer.utils.FileUtil;
import com.baijiayun.bjhl_liveplayer.widgets.TextEditFrameLayout;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPDocumentModel;
import com.baijiayun.livecore.models.LPUploadDocModel;
import com.baijiayun.livecore.ppt.PPTView;
import com.baijiayun.livecore.ppt.listener.SimplePPTStatusListener;
import com.baijiayun.livecore.ppt.whiteboard.OnDoubleTapListener2;
import com.baijiayun.livecore.ppt.whiteboard.shape.Shape;

import java.util.Objects;

public class PPTActivity extends BaseActivity implements SelectColorFragment.SelectColorListener {
    private PPTView pptView;
    private TextView tvEditMode;
    private TextView tvShapeMode;
    private Button currentColor;
    private Button eraseSelectedShapeBt;
    private TextEditFrameLayout textEditFrameLayout;
    private FrameLayout textEditContainerFl;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_menu_ppt;
    }

    @Override
    protected void subscribe() {
        pptView.addLifecycle(getLifecycle());
        // 支持滑动翻页
        pptView.setFlingEnable(true);
        pptView.attachLiveRoom(liveRoom);
        pptView.setPPTStatusListener(new SimplePPTStatusListener() {
            @Override
            public void onPPTViewAttached() {
                pptView.setPPTTurnPagesAuth(true);
            }
        });

        pptView.setOnViewTapListener((view, x, y) -> {
            showToast("单击PPT," + "点击的坐标为x:" + x + ",y:" + y);
            if (pptView.getPPTShapeType() == LPConstants.ShapeType.Text) {
                textEditContainerFl.removeAllViews();
                textEditContainerFl.addView(textEditFrameLayout.getView());
            }
        });

        pptView.setOnDoubleTapListener(new OnDoubleTapListener2() {
            @Override
            public void onDoubleTapConfirmed() {
                showToast("双击PPT");
            }

            @Override
            public void onDoubleTapOnShape(Shape shape) {
                showToast(shape.id);
            }
        });
    }

    @Override
    protected void initView() {
        pptView = findViewById(R.id.main_menu_ppt_view);
        tvEditMode = findViewById(R.id.tv_edit_mode);
        tvShapeMode = findViewById(R.id.tv_shape_mode);
        currentColor = findViewById(R.id.current_color);
        eraseSelectedShapeBt = findViewById(R.id.erase_selected_shape);
        textEditContainerFl = findViewById(R.id.textEditContainer);

        textEditFrameLayout = new TextEditFrameLayout(this, map -> {
            if (Objects.equals(map.get("end"), "true")) {
                pptView.invalidateCurrentPage();
                textEditContainerFl.removeAllViews();
            } else {
                pptView.sendDrawTextConfirmed(map.get("beforeText"), map.get("afterText"));
            }
        });

        tvEditMode.setText("当前PPTEditMode为: " + pptView.getPPTEditMode());

        setOnClickListener(R.id.ppt_shape_mode, view -> {
            hideKeyBoard();
            pptView.setPPTEditMode(LPConstants.PPTEditMode.ShapeMode);
            pptView.setShapeType(LPConstants.ShapeType.Doodle);
            setEnable(R.id.select_color, true);
            setEnable(R.id.marked_doodle, true);
            setEnable(R.id.point, true);
            setEnable(R.id.edit_text, true);
            tvShapeMode.setVisibility(View.VISIBLE);
            tvEditMode.setText("当前PPTEditMode为: " + pptView.getPPTEditMode());
            tvShapeMode.setText("当前PPTShapeMode为: " + pptView.getPPTShapeType());
        });

        setOnClickListener(R.id.ppt_erase_mode, view -> {
            hideKeyBoard();
            pptView.setPPTEditMode(LPConstants.PPTEditMode.EraserMode);
            setEnable(R.id.select_color, false);
            setEnable(R.id.marked_doodle, false);
            setEnable(R.id.point, false);
            setEnable(R.id.text, false);
            tvEditMode.setText("当前PPTEditMode为: " + pptView.getPPTEditMode());
            tvShapeMode.setVisibility(View.GONE);
        });

        setOnClickListener(R.id.select_color, view -> {
            hideKeyBoard();
            showSelectColorFragment();
        });

        setOnClickListener(R.id.ppt_normal_mode, view -> {
            hideKeyBoard();
            pptView.setPPTEditMode(LPConstants.PPTEditMode.Normal);
            setEnable(R.id.select_color, false);
            setEnable(R.id.marked_doodle, false);
            setEnable(R.id.point, false);
            setEnable(R.id.edit_text, false);
            setEnable(R.id.erase_selected_shape, false);
            tvEditMode.setText("当前PPTEditMode为: " + pptView.getPPTEditMode());
            tvShapeMode.setVisibility(View.GONE);
        });

        setOnClickListener(R.id.next_page, view -> {
            hideKeyBoard();
            goToNextPPTPage(pptView);
        });

        setOnClickListener(R.id.pre_page, view -> {
            hideKeyBoard();
            goToPrePPTPage(pptView);
        });

        setOnClickListener(R.id.marked_doodle, view -> {
            hideKeyBoard();
            pptView.setShapeType(LPConstants.ShapeType.MarkerDoodle);
            tvShapeMode.setText("当前PPTShapeMode为: " + pptView.getPPTShapeType());

        });

        setOnClickListener(R.id.point, view -> {
            hideKeyBoard();
            pptView.setShapeType(LPConstants.ShapeType.Point);
            tvShapeMode.setText("当前PPTShapeMode为: " + pptView.getPPTShapeType());
        });

        setOnClickListener(R.id.edit_text, view -> {
            hideKeyBoard();
            pptView.setShapeType(LPConstants.ShapeType.Text);
            tvShapeMode.setText("当前PPTShapeMode为: " + pptView.getPPTShapeType());
        });

        setOnClickListener(R.id.erase_all, view -> {
            hideKeyBoard();
            pptView.eraseAllShapes();
        });

        setOnClickListener(R.id.upload_image_button, view -> {
            hideKeyBoard();
            Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
            intent.setType("image/*");
            startActivityForResult(intent, 0);
        });

        setOnClickListener(R.id.upload_document_button, view -> {
            hideKeyBoard();
            Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
            intent.setType("*/*");
            startActivityForResult(intent, 1);
        });

        setOnClickListener(R.id.disable_zoom_button, view -> {
            hideKeyBoard();
            pptView.setZoomable(false);
        });

        setOnClickListener(R.id.enable_zoom_button, view -> {
            hideKeyBoard();
            pptView.setZoomable(true);
        });

        setOnClickListener(R.id.select_mode, view -> {
            hideKeyBoard();
            pptView.setPPTEditMode(LPConstants.PPTEditMode.SelectMode);
            eraseSelectedShapeBt.setEnabled(true);
        });

        eraseSelectedShapeBt.setOnClickListener(view -> {
            hideKeyBoard();
            pptView.eraseShapes();
        });

        if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            findViewById(R.id.disable_over_teacher_page_tv).setVisibility(View.VISIBLE);
            findViewById(R.id.disable_over_teacher_page).setVisibility(View.VISIBLE);
            findViewById(R.id.enable_over_teacher_page).setVisibility(View.VISIBLE);
        }

        setOnClickListener(R.id.disable_over_teacher_page, view -> {
            liveRoom.getDocListVM().setDisableOverTeacherMaxPage(true);
        });

        setOnClickListener(R.id.enable_over_teacher_page, view -> {
            liveRoom.getDocListVM().setDisableOverTeacherMaxPage(false);
        });
    }

    private void hideKeyBoard() {
        if (textEditContainerFl.getChildCount() != 0) {
            pptView.invalidateCurrentPage();
            textEditContainerFl.removeAllViews();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        Uri uri;
        if (resultCode != RESULT_OK) {
            return;
        }
        switch (requestCode) {
            case 0:
                uri = data.getData();
                String realPathFromURI = FileUtil.getPath(this, uri);
                liveRoom.getDocListVM().uploadImageWithProgress(realPathFromURI)
                        .subscribe(mergedModel -> {
                            LPUploadDocModel lpUploadDocModel = mergedModel.response;
                            LPDocumentModel documentModel = LPDocumentModel.getImgModelWithUserInfo(
                                    String.valueOf(lpUploadDocModel.fileId), lpUploadDocModel.fext, lpUploadDocModel.name
                                    , lpUploadDocModel.width, lpUploadDocModel.height, lpUploadDocModel.url,
                                    liveRoom.getCurrentUser().getName(), liveRoom.getCurrentUser().getNumber(), liveRoom.getCurrentUser().getType());
                            documentModel.size = lpUploadDocModel.size;
                            liveRoom.getDocListVM().addPictureDocument(documentModel);
                        });
                break;
            case 1:
                uri = data.getData(); // 获取用户选择文件的URI
                String path = FileUtil.getPath(this, uri);
                showToast(path);
                liveRoom.getDocListVM().uploadPPTWithProgress(path, false)
                        .subscribe(mergedModel -> {
                            LPUploadDocModel lpUploadDocModel = mergedModel.response;
                            LPDocumentModel documentModel = LPDocumentModel.getImgModelWithUserInfo(
                                    String.valueOf(lpUploadDocModel.fileId), lpUploadDocModel.fext, lpUploadDocModel.name
                                    , lpUploadDocModel.width, lpUploadDocModel.height, lpUploadDocModel.url,
                                    liveRoom.getCurrentUser().getName(), liveRoom.getCurrentUser().getNumber(), liveRoom.getCurrentUser().getType());
                            documentModel.size = lpUploadDocModel.size;
                            liveRoom.getDocListVM().addPictureDocument(documentModel);
                        });
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.select_shape_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        hideKeyBoard();
        if (pptView.getPPTEditMode() != LPConstants.PPTEditMode.ShapeMode) {
            showToast("当前不是画笔模式,请设置成画笔模式后再选择");
        }
        switch (item.getItemId()) {
            case R.id.straight_line:
                pptView.setShapeType(LPConstants.ShapeType.StraightLine);
                showToast(item.getTitle().toString());
                break;
            case R.id.arrow:
                pptView.setShapeType(LPConstants.ShapeType.Arrow);
                showToast(item.getTitle().toString());
                break;
            case R.id.double_arrow:
                pptView.setShapeType(LPConstants.ShapeType.DoubleArrow);
                showToast(item.getTitle().toString());
                break;
            case R.id.rectangle:
                pptView.setShapeType(LPConstants.ShapeType.Rect);
                showToast(item.getTitle().toString());
                break;
            case R.id.oval:
                pptView.setShapeType(LPConstants.ShapeType.Oval);
                showToast(item.getTitle().toString());
                break;
            case R.id.triangle:
                pptView.setShapeType(LPConstants.ShapeType.Triangle);
                showToast(item.getTitle().toString());
                break;
            case R.id.rectangle_solid:
                pptView.setShapeType(LPConstants.ShapeType.RectSolid);
                showToast(item.getTitle().toString());
                break;
            case R.id.oval_solid:
                pptView.setShapeType(LPConstants.ShapeType.OvalSolid);
                showToast(item.getTitle().toString());
                break;
            case R.id.triangle_solid:
                pptView.setShapeType(LPConstants.ShapeType.TriangleSolid);
                showToast(item.getTitle().toString());
                break;
            case R.id.dotted_line:
                pptView.setShapeType(LPConstants.ShapeType.DottedLine);
                showToast(item.getTitle().toString());
                break;
            default:
                break;
        }
        return true;
    }

    public void goToNextPPTPage(PPTView pptView) {
        if (liveRoom.getDocListVM().isDisableOverTeacherMaxPage()
                && liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Student
                && pptView.getCurrentPageIndex() >= pptView.getMaxPage()) {
            showToast("到达老师当前页");
            return;
        }
        int maxPage = pptView.getTotalPageNumber();
        if (pptView.getCurrentPageIndex() == maxPage - 1) {
            showToast("当前已经是最后一页");
            return;
        }
        pptView.gotoNextPage();
    }

    public void goToPrePPTPage(PPTView pptView) {
        if (pptView.getCurrentPageIndex() == 0) {
            showToast("当前是第一页");
            return;
        }
        pptView.gotoPrevPage();
    }

    private void showSelectColorFragment() {
        SelectColorFragment selectColorFragment = new SelectColorFragment();
        selectColorFragment.show(getSupportFragmentManager(), "selectColor");
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    public void setSelectedColor(DialogFragment fragment, int color) {
        pptView.setPaintColor(color);
        currentColor.setBackgroundColor(color);
        fragment.getDialog().cancel();
    }

}