package com.baijiayun.bjhl_liveplayer.activity;

import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjhl_liveplayer.fragment.OperateVideoFragment;
import com.baijiayun.bjyrtcengine.Defines.BJYRtcCommon;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.livecore.viewmodels.impl.LPSpeakQueueViewModel;
import com.baijiayun.livecore.wrapper.LPPlayer;
import com.baijiayun.livecore.wrapper.LPRecorder;
import com.baijiayun.livecore.wrapper.impl.LPCameraView;
import com.baijiayun.livecore.wrapper.impl.LPVideoView;

import io.reactivex.android.schedulers.AndroidSchedulers;

public class PullStreamActivity extends BaseActivity implements OperateVideoFragment.VideoOperationListener {

    private static final String TAG = "pullStream";

    private LPPlayer lpPlayer;
    private LPCameraView lpCameraView;
    private LPRecorder recorder;
    protected ViewGroup.LayoutParams layoutParams;
    private LPVideoView mixStreamVideoView;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_player;
    }

    LinearLayout remoteVideoContainerLl;
    FrameLayout localVideoContainerFl;
    TextView remoteVideoDefinitionTv;
    TextView remoteVideoStateTv;
    TextView remoteAudioStateTv;

    @Override
    protected void initView() {
        remoteVideoContainerLl = findViewById(R.id.main_menu_player_remote_video);
        localVideoContainerFl = findViewById(R.id.main_menu_player_local_video);
        remoteVideoDefinitionTv = findViewById(R.id.remote_video_definition);
        remoteVideoStateTv = findViewById(R.id.remote_video_state);
        remoteAudioStateTv = findViewById(R.id.remote_audio_state);

        initRemoteVideoParams();

        setOnClickListener(R.id.mute_all_remote_video, view -> {
            lpPlayer.muteAllRemoteVideo(true);
            showToast("关闭所有视频");
        });

        setOnClickListener(R.id.mute_all_remote_audio, view -> {
            lpPlayer.muteAllRemoteAudio(true);
            showToast("关闭所有音频");
        });

        setOnClickListener(R.id.open_all_remote_video, view -> {
            lpPlayer.muteAllRemoteVideo(false);
            showToast("打开所有视频");
        });

        setOnClickListener(R.id.open_all_remote_audio, view -> {
            lpPlayer.muteAllRemoteAudio(false);
            showToast("打开所有音频");
        });

        setOnClickListener(R.id.publish_local_video, view -> {
            pushLocalStream();
        });

        setOnClickListener(R.id.stop_publish_local_video, view -> {
            if (recorder != null) {
                recorder.stopPublishing();
                showToast("下麦，恢复合流模式");
            }
            localVideoContainerFl.removeAllViews();
        });

        setOnClickListener(R.id.speak_apply_button, view -> {
            liveRoom.getSpeakQueueVM().requestSpeakApply();
        });
    }

    private void pushLocalStream() {
        recorder = liveRoom.getRecorder();
        if (recorder.isPublishing()) {
            return;
        }
        lpCameraView = new LPCameraView(this);
        lpCameraView.setViewType(LPConstants.LPVideoViewType.SURFACE_VIEW);
        lpCameraView.setAspectRatio(LPConstants.LPAspectRatio.Fit);
        localVideoContainerFl.addView(lpCameraView, layoutParams);
        recorder.setPreview(lpCameraView);
        recorder.attachVideo();
        recorder.attachAudio();
        recorder.publish();
        showToast("上麦，停止拉合流走webrtc拉流");
        // 上麦后停止拉合流，转为走webrtc低延时拉流
        lpPlayer.playAVClose(LPSpeakQueueViewModel.FAKE_MIX_STREAM_USER_ID);
        LPVideoView toRemoveVideoView = getRemoteVideoView(LPSpeakQueueViewModel.FAKE_MIX_STREAM_USER_ID);
        if (toRemoveVideoView != null) {
            remoteVideoContainerLl.removeView(toRemoveVideoView);
        }
    }

    private void initRemoteVideoParams() {
        int height = getResources().getDimensionPixelSize(R.dimen.remote_video_height);
        float ratio = 4f / 3;
        int width = (int) (height * ratio);
        layoutParams = new ViewGroup.LayoutParams(width, height);
    }

    @Override
    protected void subscribe() {
        lpPlayer = liveRoom.getPlayer();

        compositeDisposable.add(liveRoom.getSpeakQueueVM()
                .getObservableOfMediaPublish()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    Log.e(TAG, "media_publish " + iMediaModel.toString());
                    handleMediaPublish(iMediaModel);
                }));

        compositeDisposable.add(liveRoom.getSpeakQueueVM()
                .getObservableOfMixModeMediaPublish()
                .subscribe(iMediaModel -> {
                    Log.d(TAG, "合流下远端音视频发生了变化 " + iMediaModel.toString());
                    if (iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainScreenShare) {
                        if (iMediaModel.isVideoOn()) {
                            Log.e(TAG, "合流模式下，开启屏幕共享");
                            mixStreamVideoView.setAspectRatio(LPConstants.LPAspectRatio.Fit);
                        } else {
                            Log.e(TAG, "合流模式下，关闭屏幕共享");
                            mixStreamVideoView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
                        }
                        lpPlayer.playAVClose(LPSpeakQueueViewModel.FAKE_MIX_STREAM_USER_ID);
                        lpPlayer.playVideo(LPSpeakQueueViewModel.FAKE_MIX_STREAM_USER_ID, mixStreamVideoView);
                    }
                }));

        compositeDisposable.add(liveRoom.getSpeakQueueVM()
                .getObservableOfActiveUsers()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModels -> {
                    if (iMediaModels.size() == 0) {
                        showToast("当前无人发言");
                    } else {
                        remoteVideoContainerLl.removeAllViews();
                        for (IMediaModel iMediaModel : iMediaModels) {
                            handleMediaPublish(iMediaModel);
                        }
                    }
                }));

        compositeDisposable.add(liveRoom.getSpeakQueueVM().getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (liveRoom.getCurrentUser().getUserId().equals(iMediaControlModel.getUser().getUserId()) && iMediaControlModel.isApplyAgreed()) {
                        pushLocalStream();
                    }
                }));
    }

    /**
     * 处理拉流信息
     *
     * @param mediaModel
     */
    private void handleMediaPublish(IMediaModel mediaModel) {
        LPVideoView remoteVideoView = getRemoteVideoView(mediaModel.getMediaId());
        if (remoteVideoView != null) {
            lpPlayer.playAVClose(mediaModel.getMediaId());
            if (mediaModel.isVideoOn()) {
                lpPlayer.playVideo(mediaModel.getMediaId(), remoteVideoView);
            } else if (mediaModel.isAudioOn()) {
                lpPlayer.playAudio(mediaModel.getMediaId());
            } else {
                lpPlayer.playAVClose(mediaModel.getMediaId());
                View toRemoveVideoView = getRemoteVideoView(mediaModel.getMediaId());
                if (toRemoveVideoView != null) {
                    remoteVideoContainerLl.removeView(toRemoveVideoView);
                }
            }
        } else {
            if (mediaModel.isVideoOn() || mediaModel.isAudioOn()) {
                remoteVideoView = createRemoteVideoView();
                remoteVideoView.setTag(mediaModel.getMediaId());
                remoteVideoView.setOnClickListener(view -> {
                    OperateVideoFragment operateVideoFragment = new OperateVideoFragment(mediaModel);
                    operateVideoFragment.show(getSupportFragmentManager(), "operateVideo");
                });
                remoteVideoContainerLl.addView(remoteVideoView, layoutParams);
                if (mediaModel.isVideoOn()) {
                    lpPlayer.playVideo(mediaModel.getMediaId(), remoteVideoView);
                } else {
                    lpPlayer.playAudio(mediaModel.getMediaId());
                }
            }
        }
    }

    private LPVideoView createRemoteVideoView() {
        LPVideoView lpVideoView;
        lpVideoView = new LPVideoView(this);
        lpVideoView.setViewType(LPConstants.LPVideoViewType.SURFACE_VIEW);
        lpVideoView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
        return lpVideoView;
    }


    private LPVideoView getRemoteVideoView(String mediaId) {
        for (int i = 0; i < remoteVideoContainerLl.getChildCount(); i++) {
            View childView = remoteVideoContainerLl.getChildAt(i);
            if ((childView instanceof LPVideoView) && childView.getTag().equals(mediaId)) {
                return (LPVideoView) childView;
            }
        }
        return null;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (recorder != null) {
            recorder.stopPublishing();
        }
        lpPlayer.release();
    }

    @Override
    public void openVideo(IMediaModel iMediaModel) {
        lpPlayer.muteRemoteVideo(iMediaModel.getMediaId(), false);
    }

    @Override
    public void closeVideo(IMediaModel iMediaModel) {
        lpPlayer.muteRemoteVideo(iMediaModel.getMediaId(), true);
    }

    @Override
    public void openAudio(IMediaModel iMediaModel) {
        lpPlayer.muteRemoteAudio(iMediaModel.getMediaId(), false);
    }

    @Override
    public void closeAudio(IMediaModel iMediaModel) {
        lpPlayer.muteRemoteAudio(iMediaModel.getMediaId(), true);
    }

    @Override
    public void pullHighVideoStream(IMediaModel iMediaModel) {
        lpPlayer.setRemoteVideoStreamType(iMediaModel.getMediaId(), BJYRtcCommon.DualStreamType.VIDEO_STREAM_HIGH);
    }

    @Override
    public void pullLowVideoStream(IMediaModel iMediaModel) {
        lpPlayer.setRemoteVideoStreamType(iMediaModel.getMediaId(), BJYRtcCommon.DualStreamType.VIDEO_STREAM_LOW);
    }


}