package com.baijiayun.bjhl_liveplayer.activity;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;
import android.webkit.JavascriptInterface;
import android.webkit.WebChromeClient;
import android.webkit.WebResourceError;
import android.webkit.WebResourceRequest;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import androidx.annotation.RequiresApi;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPJsonModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.LiveSDK;
import com.baijiayun.livecore.utils.CommonUtils;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;


public class QuizActivity extends BaseActivity {
    private static final String TAG = "QuizActivity";
    private static final String ARGS_TYPE = "1";
    private static final String WINDOW_NAME = "bjlapp";
    private boolean isUrlLoaded;// 默认false
    private List<LPJsonModel> signalList = new ArrayList<>();
    private IUserModel currentUserInfo;
    private String quizId;
    private String roomId;
    private String roomToken;
    WebView webView;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_quiz;
    }

    @Override
    protected void subscribe() {
//        开始测验
        compositeDisposable.add(liveRoom.getQuizVM().getObservableOfQuizStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::onQuizArrived));

        compositeDisposable.add(liveRoom.getQuizVM().getObservableOfQuizEnd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpJsonModel -> {
                    if (liveRoom.getCurrentUser().getType() != LPConstants.LPUserType.Teacher) {
                        onEndArrived(lpJsonModel);
//                        更新正在答题的缓存
                        liveRoom.getQuizVM().sendQuizReq();
                    }
                }));

        //正在答题回调
        compositeDisposable.add(liveRoom.getQuizVM().getObservableOfQuizRes()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpJsonModel -> {
                    if (liveRoom.getCurrentUser().getType() != LPConstants.LPUserType.Teacher) {
                        if (lpJsonModel != null && lpJsonModel.data != null) {
                            String id = "";
                            if (lpJsonModel.data.has("quiz_id")) {
                                id = lpJsonModel.data.get("quiz_id").getAsString();
                            }
                            // 没有solution、"solution":{}、"solution":"null"
                            boolean solutionStatus = !lpJsonModel.data.has("solution") || lpJsonModel.data.getAsJsonObject("solution").entrySet().isEmpty()
                                    || lpJsonModel.data.getAsJsonObject("solution").isJsonNull();
                            boolean endFlag = lpJsonModel.data.get("end_flag").getAsInt() == 1;
                            //quizId非空、solution是空、没有结束答题 才弹窗
                            if (!TextUtils.isEmpty(id) && solutionStatus && !endFlag) {
                                onQuizArrived(lpJsonModel);
                            }
                        }
                    }
                }));

        //转发答案回调
        compositeDisposable.add(liveRoom.getQuizVM().getObservableOfQuizSolution()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpJsonModel -> {
                    if (liveRoom.getCurrentUser().getType() != LPConstants.LPUserType.Teacher) {
                        onQuizArrived(lpJsonModel);
                    }
                }));

    }


    private void onQuizArrived(LPJsonModel lpJsonModel) {
        roomId = lpJsonModel.data.get("class_id").getAsString();
        quizId = lpJsonModel.data.get("quiz_id").getAsString();
        roomToken = liveRoom.getQuizVM().getRoomToken();
        onGetCurrentUser();
        onArrived(lpJsonModel);
    }

    public void onGetCurrentUser() {
        this.currentUserInfo = liveRoom.getCurrentUser();
    }

    @SuppressLint({"JavascriptInterface", "WrongConstant"})
    @Override
    protected void initView() {
        webView = mRootView.findViewById(R.id.answer_webview);
//        在当前webView中显示
        webView.getSettings().setJavaScriptEnabled(true);
        webView.addJavascriptInterface(this, WINDOW_NAME);
        webView.setVerticalScrollBarEnabled(false);
        webView.getSettings().setDomStorageEnabled(true);
        webView.getSettings().setAllowFileAccess(true);
        webView.getSettings().setCacheMode(WebSettings.LOAD_DEFAULT);
        webView.setWebChromeClient(new WebChromeClient() {

        });
        webView.setWebViewClient(new WebViewClient() {
            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);
                isUrlLoaded = true;
                callJsInQueue();
            }

            @Override
            public void onPageStarted(WebView view, String url, Bitmap favicon) {
                super.onPageStarted(view, url, favicon);
                isUrlLoaded = false;
            }

            @Override
            public void onReceivedSslError(WebView view, android.webkit.SslErrorHandler handler, android.net.http.SslError error) {
                handler.proceed();

            }

            @RequiresApi(api = Build.VERSION_CODES.M)
            @Override
            public void onReceivedError(WebView view, WebResourceRequest request, WebResourceError error) {
                super.onReceivedError(view, request, error);
                if (error.getErrorCode() != -11) {
                    Log.e(TAG, "onReceivedError: " + error.getErrorCode() + "--" + error.getDescription());
                }
            }
        });
    }

    /******************
     * js call android
     ******************/
    @JavascriptInterface
    public void close() {
        Toast.makeText(getApplicationContext(), "关闭答题界面", Toast.LENGTH_SHORT).show();
    }

    @JavascriptInterface
    public void sendMessage(String json) {
        liveRoom.getQuizVM().sendSubmit(json);
    }

    private void callJsInQueue() {
        if (!signalList.isEmpty()) {
            for (LPJsonModel jsonModel : signalList) {
                callJs(jsonModel.data.toString());
            }
            signalList.clear();
        }
    }

    private void callJs(String json) {
        webView.loadUrl("javascript:bjlapp.receivedMessage(" + json + ")");
    }

    private void loadUrl() {
        try {
            roomToken = URLEncoder.encode(roomToken, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        String params = "?userNumber=" + currentUserInfo.getNumber() + "&userName=" + CommonUtils.encodeUTF8(currentUserInfo.getName()) + "&quizId=" + quizId
                + "&roomId=" + roomId + "&token=" + roomToken + "&argType=" + ARGS_TYPE + "&userGroup=" + currentUserInfo.getGroup() + "&skin=dark";
        String host = LPConstants.HOSTS_WEB[LiveSDK.getDeployType().getType()];
        String url = host.concat("m/quiz/student").concat(params);
        LPLogger.i(getClass().getSimpleName() + " : " + url);
        webView.loadUrl(url);
    }

    public void onArrived(LPJsonModel jsonModel) {
        if (jsonModel.data.has("quiz_id")) {
            quizId = jsonModel.data.get("quiz_id").getAsString();
        } else {
            quizId = "";
        }
        if (jsonModel.data.has("class_id")) {
            roomId = jsonModel.data.get("class_id").getAsString();
        } else {
            roomId = "";
        }
        if (isUrlLoaded) {
            //如果页面已经加载，那么只需要加载js
            callJs(jsonModel.data.toString());
        } else {
            //如果页面没有加载，那就先加载页面，然后将要加载的js逻辑加入队列，当页面加载完成后会处理队列
            loadUrl();
            signalList.add(jsonModel);
        }
    }

    @Override
    public void onDestroy() {
        webView.destroy();
        signalList.clear();
        super.onDestroy();
    }

    public void onEndArrived(LPJsonModel jsonModel) {
        String key = jsonModel.data.get("message_type").getAsString();
        if (!"quiz_end".equals(key)) {
            return;
        }
        /*只有load完之后才去转发h5，否则加入队列，load完成后依次转发*/
        if (isUrlLoaded) {
            callJs(jsonModel.data.toString());
        } else {
            signalList.add(jsonModel);
        }
    }
}