package com.baijiayun.bjhl_liveplayer.activity;

import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.bjyrtcengine.Defines.BJYRtcCommon;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.wrapper.LPPlayer;
import com.baijiayun.livecore.wrapper.LPRecorder;
import com.baijiayun.livecore.wrapper.impl.LPCameraView;
import com.baijiayun.livecore.wrapper.impl.LPVideoView;

import io.reactivex.android.schedulers.AndroidSchedulers;

public class RecorderActivity extends BaseActivity {
    /**
     * recorder用于发布本地音视频
     */
    private LPRecorder recorder;
    /**
     * CameraView 用于承载本地流预览
     */
    private LPCameraView lpCameraView;

    /**
     * VideoView 用于承载远端流显示
     */
    private LPVideoView lpVideoView;
    /**
     * player用于播放远程音视频流
     */
    private LPPlayer player;

    private String mediaId;

    @Override
    protected int getContentViewId() {
        return R.layout.activity_menu_recorder;
    }


    TextView localVideoDefinitionTv;
    TextView upPacketLossRateTv;
    TextView localVideoMirrorModeTv;
    Button publishBt;
    TextView localVideoStatusTv;
    TextView localAudioStatusTv;
    TextView currentCameraOrientationTv;
    TextView cameraCountTv;
    ImageView closeVideoIv;

    @Override
    protected void initView() {
        localVideoDefinitionTv = findViewById(R.id.local_video_definition);
        upPacketLossRateTv = findViewById(R.id.up_packet_loss_rate);
        localVideoMirrorModeTv = findViewById(R.id.local_mirror_mode);
        publishBt = findViewById(R.id.publish_local_video);
        localVideoStatusTv = findViewById(R.id.local_video_status);
        localAudioStatusTv = findViewById(R.id.local_audio_status);
        closeVideoIv = findViewById(R.id.main_menu_recorder_local_image);
        currentCameraOrientationTv = findViewById(R.id.current_camera_orientation);
        cameraCountTv = findViewById(R.id.camera_count);
        localVideoMirrorModeTv.setText("水平镜像");
        lpCameraView = findViewById(R.id.camera_view);

        setOnClickListener(R.id.publish_local_video, view -> {
            if (recorder.isPublishing()) {
                return;
            }
            lpCameraView.setViewType(LPConstants.LPVideoViewType.SURFACE_VIEW);
            lpCameraView.setAspectRatio(LPConstants.LPAspectRatio.Fit);
            recorder.setPreview(lpCameraView);
            recorder.attachAVideo();
            recorder.publish();
        });

        setOnClickListener(R.id.local_video_fit, view -> {
            lpCameraView.setAspectRatio(LPConstants.LPAspectRatio.Fit);
            recorder.stopLocalPreview();
            recorder.startLocalPreview(lpCameraView);
        });

        setOnClickListener(R.id.local_video_fill, view -> {
            lpCameraView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
            recorder.stopLocalPreview();
            recorder.startLocalPreview(lpCameraView);
        });

        setOnClickListener(R.id.switch_camera, view -> {
            switchCameraOrientation();
        });

        setOnClickListener(R.id.open_local_video, view -> {
            openLocalVideo();
        });

        setOnClickListener(R.id.close_local_video, view -> {
            closeLocalVideo();
        });

        setOnClickListener(R.id.open_local_audio, view -> {
            openLocalAudio();
        });

        setOnClickListener(R.id.close_local_audio, view -> {
            closeLocalAudio();
        });

        setOnClickListener(R.id.open_local_avideo, view -> {
            openLocalAVideo();
        });

        setOnClickListener(R.id.close_local_avideo, view -> {
            closeLocalAVideo();
        });


        setOnClickListener(R.id.resolution_low, view -> {
            recorder.setCaptureVideoDefinition(LPConstants.LPResolutionType.LOW);
            setLocalDefinitionChange(LPConstants.LPResolutionType.LOW);
        });

        setOnClickListener(R.id.resolution_high, view -> {
            recorder.setCaptureVideoDefinition(LPConstants.LPResolutionType.HIGH);
            setLocalDefinitionChange(LPConstants.LPResolutionType.HIGH);
        });

        setOnClickListener(R.id.resolution_360, view -> {
            recorder.setCaptureVideoDefinition(LPConstants.LPResolutionType._360);
            setLocalDefinitionChange(LPConstants.LPResolutionType._360);
        });

        setOnClickListener(R.id.resolution_720, view -> {
            recorder.setCaptureVideoDefinition(LPConstants.LPResolutionType._720);
            setLocalDefinitionChange(LPConstants.LPResolutionType._720);
        });

        setOnClickListener(R.id.resolution_1080, view -> {
            recorder.setCaptureVideoDefinition(LPConstants.LPResolutionType._1080);
            setLocalDefinitionChange(LPConstants.LPResolutionType._1080);
        });


        setOnClickListener(R.id.auto_mirror_mode, view -> {
            recorder.setLocalVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.AUTO_MIRROR);
            recorder.setEncVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.AUTO_MIRROR);
            localVideoMirrorModeTv.setText(getMirrorText(BJYRtcCommon.VideoMirrorMode.AUTO_MIRROR));
        });

        setOnClickListener(R.id.horizontal_mirror_mode, view -> {
            recorder.setLocalVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_MIRROR);
            recorder.setEncVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_MIRROR);
            localVideoMirrorModeTv.setText(getMirrorText(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_MIRROR));
        });

        setOnClickListener(R.id.vertical_mirror_mode, view -> {
            recorder.setLocalVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.VERTICAL_MIRROR);
            recorder.setEncVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.VERTICAL_MIRROR);
            localVideoMirrorModeTv.setText(getMirrorText(BJYRtcCommon.VideoMirrorMode.VERTICAL_MIRROR));
        });

        setOnClickListener(R.id.vertical_horizontal_mirror_mode, view -> {
            recorder.setLocalVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_VERTICAL_MIRROR);
            recorder.setEncVideoMirrorMode(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_VERTICAL_MIRROR);
            localVideoMirrorModeTv.setText(getMirrorText(BJYRtcCommon.VideoMirrorMode.HORIZONTAL_VERTICAL_MIRROR));
        });

        initVideo();
    }

    private void switchCameraOrientation() {
        if (!recorder.isVideoAttached()) {
            showToast("请先打开摄像头");
            return;
        }
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if (recorder.getCameraCount() <= 1) {
            showToast("摄像头数量不够,无法切换");
            return;
        }
        recorder.switchCamera();
        if (recorder.getCameraOrientation()) {
            currentCameraOrientationTv.setText("当前摄像头:前置摄像头");
        } else {
            currentCameraOrientationTv.setText("当前摄像头:后置摄像头");
        }
    }


    @Override
    protected void subscribe() {
        recorder = liveRoom.getRecorder();
        player = liveRoom.getPlayer();
        cameraCountTv.setText(String.format("摄像头的数量:%d", recorder.getCameraCount()));

        compositeDisposable.add(liveRoom.getSpeakQueueVM()
                .getObservableOfMediaPublish()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    String publishMediaId = iMediaModel.getMediaId();
                    if (iMediaModel.isVideoOn()) {
                        player.playVideo(publishMediaId, lpVideoView);
                    } else if(!iMediaModel.isVideoOn() && !iMediaModel.isAudioOn()){
                        player.playAVClose(publishMediaId);
                    }
                    boolean videoOn = iMediaModel.isVideoOn();
                    boolean audioOn = iMediaModel.isAudioOn();
                    if (videoOn) {
                        showToast(String.format("%s打开了摄像头", iMediaModel.getUser().getName()));
                    } else {
                        showToast(String.format("%s关闭了摄像头", iMediaModel.getUser().getName()));
                    }
                }));

        compositeDisposable.add(liveRoom.getSpeakQueueVM()
                .getObservableOfActiveUsers()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModels -> {
                    if (iMediaModels.size() == 0) {
                        showToast("当前无人发言");
                    } else {
                        mediaId = iMediaModels.get(0).getMediaId();
                        player.playVideo(mediaId, lpVideoView);
                    }
                }));


        compositeDisposable.add(recorder.getObservableOfCameraOn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isCameraOn -> {
                    if (isCameraOn) {
                        showToast("摄像头打开");
                        localVideoStatusTv.setText("摄像头:开启");
                        closeVideoIv.setVisibility(View.GONE);
                        lpCameraView.setVisibility(View.VISIBLE);
                        currentCameraOrientationTv.setVisibility(View.VISIBLE);
                        if (recorder.getCameraOrientation()) {
                            currentCameraOrientationTv.setText("当前摄像头:前置摄像头");
                        } else {
                            currentCameraOrientationTv.setText("当前摄像头:后置摄像头");
                        }
                    } else {
                        showToast("摄像头关闭");
                        localVideoStatusTv.setText("摄像头:关闭");
                        closeVideoIv.setVisibility(View.VISIBLE);
                        lpCameraView.setVisibility(View.GONE);
                        currentCameraOrientationTv.setVisibility(View.GONE);
                    }
                })
        );

        compositeDisposable.add(recorder.getObservableOfMicOn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isAudioOn -> {
                    if (isAudioOn) {
                        showToast("麦克风打开");
                        localAudioStatusTv.setText("麦克风:开启");
                    } else {
                        showToast("麦克风关闭");
                        localAudioStatusTv.setText("麦克风:关闭");
                    }
                })
        );

        //上行丢包率监听
        compositeDisposable.add(recorder.getObservableOfUpPacketLossRate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(localStreamStats -> {
                    upPacketLossRateTv.setText(String.format("上行丢包率为:%d", localStreamStats.uploadLossRate));
                }));
    }


    private void initVideo() {
        // region 初始化远端流
        lpVideoView = new LPVideoView(this);
        lpVideoView.setViewType(LPConstants.LPVideoViewType.SURFACE_VIEW);
        lpVideoView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
        // endregion
    }

    private String getMirrorText(BJYRtcCommon.VideoMirrorMode localVideoMirrorMode) {
        String str = null;
        switch (localVideoMirrorMode) {
            case AUTO_MIRROR:
                str = "正常模式";
                break;
            case VERTICAL_MIRROR:
                str = "垂直镜像";
                break;
            case HORIZONTAL_MIRROR:
                str = "水平镜像";
                break;
            case HORIZONTAL_VERTICAL_MIRROR:
                str = "中心对称镜像";
                break;
            default:
                break;
        }
        return str;
    }


    private void setLocalDefinitionChange(LPConstants.LPResolutionType type) {
        switch (type) {
            case LOW:
                localVideoDefinitionTv.setText("当前分辨率为320*240");
                break;
            case HIGH:
                localVideoDefinitionTv.setText("当前分辨率为640*480");
                break;
            case _360:
                localVideoDefinitionTv.setText("当前分辨率为480*360");
                break;
            case _720:
                localVideoDefinitionTv.setText("当前分辨率为1280*720");
                break;
            case _1080:
                localVideoDefinitionTv.setText("当前分辨率为1920*1080");
                break;
            default:
                break;
        }
    }


    //打开本地视频
    private void openLocalVideo() {
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if (recorder.isVideoAttached()) {
            return;
        }

        recorder.attachVideo();
        recorder.setPreview(lpCameraView);
    }

    //关闭本地视频
    private void closeLocalVideo() {
        if (!recorder.isVideoAttached()) {
            showToast("摄像头未打开");
            return;
        }
        recorder.detachVideo();
    }

    //打开本地音频
    private void openLocalAudio() {
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if (recorder.isAudioAttached()) {
            return;
        }
        recorder.attachAudio();
    }

    //关闭本地音频
    private void closeLocalAudio() {
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if (!recorder.isAudioAttached()) {
            showToast("麦克风未打开");
            return;
        }
        recorder.detachAudio();
    }

    private void openLocalAVideo() {
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if (recorder.isVideoAttached() || recorder.isAudioAttached()) {
            showToast("将音视频都关闭后再尝试");
            return;
        }
        recorder.attachAVideo();
        recorder.setPreview(lpCameraView);
    }

    private void closeLocalAVideo() {
        if (!recorder.isPublishing()) {
            showToast("请先开始推流");
            return;
        }
        if ((!recorder.isAudioAttached()) || (!recorder.isVideoAttached())) {
            showToast("将音视频都打开后再尝试");
            return;
        }
        recorder.detachAVideo();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        recorder.stopPublishing();
        if (mediaId == null) {
            return;
        }
        player.playAVClose(mediaId);
    }
}