package com.baijiayun.bjhl_liveplayer.activity.toolbox

import android.app.AlertDialog
import android.app.ProgressDialog
import android.content.DialogInterface
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.baijiayun.bjhl_liveplayer.R
import com.baijiayun.bjhl_liveplayer.activity.BaseActivity
import com.baijiayun.livecore.models.*
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import java.util.*
import java.util.concurrent.TimeUnit

class AnswerActivity : BaseActivity() {
    companion object{
        const val TAG: String= "AnswerActivity"
    }
    private var submitQuestion: LPAnswerModel? = null
    private lateinit var submitAnswer: LPAnswerModel
    private lateinit var sheetOptions: MutableList<LPAnswerSheetOptionModel>

    private var inputQuestion: EditText? = null
    private var answerOptions: RelativeLayout? = null
    private var optionsJudge: ViewGroup? = null
    private var optionsChoose: ViewGroup? = null
    private var answerCountTimer: Disposable? = null
    private lateinit var teacherDialog: ProgressDialog
    private lateinit var studentDialog: ProgressDialog
    private lateinit var stuAlert: AlertDialog
    private lateinit var stuBuilder: AlertDialog.Builder
    private var progress: Long = 0
    private var showStuName: TextView? = null
    private var showStuSelected: TextView? = null
    private var showStuisCorrect: TextView? = null

    override fun getContentViewId(): Int {
        return R.layout.activity_answer
    }

    override fun subscribe() {
        //发布答题监听：应该是学生和老师都有响应，只有老师能够发起回调
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfAnswerStart
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpAnswerModel: LPAnswerModel ->
                logUtil("==========发布答题返回==========")
                progress = lpAnswerModel.duration
                submitAnswer = lpAnswerModel
                sheetOptions = lpAnswerModel.options
                if (liveRoom.isTeacherOrAssistant) {
                    //设置ProgressDialog
                    teacherDialog.setTitle("结束倒计时:" + progress + "s")
                    teacherDialog.setMessage("" + submitAnswer.getDescription())
                    teacherDialog.setCancelable(false)
                    teacherDialog.setButton(
                        DialogInterface.BUTTON_NEGATIVE, "撤销"
                    ) { _: DialogInterface?, _: Int ->
                        //发送撤销信令
                        logUtil("**********发送撤销信令**********")
                        liveRoom.toolBoxVM.requestAnswerEnd(true, progress)
                        answerCountTimer!!.dispose()
                    }
                    teacherDialog.setButton(
                        DialogInterface.BUTTON_POSITIVE, "结束"
                    ) { _: DialogInterface?, _: Int ->
                        //发送结束信令
                        logUtil("**********发送结束信令**********")
                        liveRoom.toolBoxVM.requestAnswerEnd(false, progress)
                        answerCountTimer!!.dispose()
                    }
                    teacherDialog.show()
                    startAnswerTimerTeacher()
                } else {
                    if (submitAnswer.isJudgement) {
                        //判断题
                        studentDialog = ProgressDialog(this)
                        studentDialog.setCancelable(false)
                        studentDialog.setTitle("回答倒计时：$progress")
                        studentDialog.setMessage(submitAnswer.description)
                        studentDialog.setButton(DialogInterface.BUTTON_POSITIVE, sheetOptions[0].text) { _: DialogInterface?, _: Int ->
                            //√
                            val lp = sheetOptions[0]
                            lp.isActive = true
                            (sheetOptions as MutableList<LPAnswerSheetOptionModel>?)!![0] = lp
                            submitAnswer.options = sheetOptions
                            liveRoom.toolBoxVM.submitAnswers(submitAnswer)
                            answerCountTimer!!.dispose()
                        }
                        studentDialog.setButton(DialogInterface.BUTTON_NEGATIVE, sheetOptions[1].text) { _: DialogInterface?, _: Int ->
                            //×
                            val lp = sheetOptions[1]
                            lp.isActive = true
                            (sheetOptions as MutableList<LPAnswerSheetOptionModel>?)!!.set(1, lp)
                            submitAnswer.options = sheetOptions
                            liveRoom.toolBoxVM.submitAnswers(submitAnswer)
                            answerCountTimer!!.dispose()
                        }
                        studentDialog.show()
                    } else {
                        //选择题
                        stuBuilder = AlertDialog.Builder(this)
                        stuBuilder.setCancelable(false)
                        stuBuilder.setTitle(
                            """
                        回答倒计时：${progress}s
                        ${submitAnswer.description}
                        """.trimIndent()
                        )
                        val list =
                            arrayOfNulls<CharSequence>(sheetOptions.size)
                        val boolSelects = BooleanArray(sheetOptions.size)
                        for (i in sheetOptions.indices) {
                            list[i] = sheetOptions.get(i).text
                        }
                        stuBuilder.setMultiChoiceItems(list, null) { _: DialogInterface?, which: Int, isChecked: Boolean ->
                            boolSelects[which] = isChecked
                        }
                        stuBuilder.setPositiveButton("提交答案") { _: DialogInterface?, _: Int ->
                            for (i in sheetOptions.indices) {
                                val lp = sheetOptions[i]
                                lp.isActive = boolSelects[i]
                                (sheetOptions as MutableList<LPAnswerSheetOptionModel>?)!![i] = lp
                            }
                            submitAnswer.options = sheetOptions

                            //toUserId: submitAnswer.userId 为目的Id即老师的id，自己的id在方法中自动获取
                            liveRoom.toolBoxVM.submitAnswers(submitAnswer)
                            answerCountTimer!!.dispose()
                        }
                        stuAlert = stuBuilder.show()
                    }
                    startAnswerTimerStudent()
                }
            })

        //撤销、停止答题监听
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfAnswerEnd
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpAnswerEndModel: LPAnswerEndModel ->
                if (liveRoom.isTeacherOrAssistant) {
                    teacherDialog.dismiss()
                    answerCountTimer!!.dispose()
                    //老师撤销答题
                    if (lpAnswerEndModel.isRevoke) {
                        logUtil("==========撤销答题==========")
                    } else {
                        logUtil("==========结束答题==========")
                        logUtil("**********请求本次答题数据**********")
                        //TODO:请求本次答题数据
                        liveRoom.toolBoxVM.requestAnswerPullReq(submitAnswer.id)
                    }
                } else {
                    //学生收到老师操作回调
                    logUtil("==========老师撤销了答题==========")
                    if (studentDialog.isShowing) {
                        studentDialog.dismiss()
                    }
                    if (stuAlert.isShowing) {
                        stuAlert.dismiss()
                    }
                    answerCountTimer!!.dispose()
                }
            })

        //答题数据更新：老师方
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfAnswerUpdate
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                logUtil(
                    "==========答题数据更新=========="
                )
            })

        //请求历史答题数据监听
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfAnswerPullRes
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { objectLPAnswerRecordModelMap: Map<Any?, LPAnswerRecordModel?> ->
                logUtil("==========请求历史答题==========")
                //暂时只有一个学生:size == 1
                if (objectLPAnswerRecordModelMap.isNotEmpty()) {
                    for (i in 0 until objectLPAnswerRecordModelMap.size) {
                        //答题
                        val lp = objectLPAnswerRecordModelMap[submitAnswer.id]
                        //TODO:正常想要获取userDetails，需要得到学生的number，也就是说需要请求在线用户列表或学生列表
                        //这里使用遍历map
                        if (lp!!.userDetails != null) {
                            val set: Set<Map.Entry<Any, LPUserAnswerDetail>> =
                                lp.userDetails.entries
                            for ((_, value) in set) {
                                //学生回答
                                showStuName!!.text = value.user.name
                                val choices = value.choices
                                val stringBuilder = StringBuilder()
                                stringBuilder.append("学生选择了：")
                                for (j in choices.indices) {
                                    stringBuilder.append(choices[j])
                                }
                                showStuSelected!!.text = stringBuilder.toString()
                                if (value.isCorrect) {
                                    showStuisCorrect!!.text = "回答正确"
                                } else {
                                    showStuisCorrect!!.text = "回答错误"
                                }
                            }
                        } else {
                            showStuName!!.text = ""
                            showStuSelected!!.text = ""
                            showStuisCorrect!!.text = ""
                        }
                    }
                }
            })
    }

    override fun initView() {
        if (liveRoom.isTeacherOrAssistant) {
            logUtil("加载老师发布的答题器页面")
            teacherDialog = ProgressDialog(this)
            //初始化AnswerModel
            submitQuestion = LPAnswerModel()
            inputQuestion = findViewById(R.id.answer_module_question_et)
            answerOptions =findViewById(R.id.answer_options)
            showStuName = findViewById(R.id.answer_stu_name)
            showStuSelected = findViewById(R.id.answer_stu_select)
            showStuisCorrect = findViewById(R.id.answer_stu_is_correct)
            optionsJudge = LayoutInflater.from(this)
                .inflate(R.layout.item_toolbox_answer_options_judge, null) as ViewGroup
            optionsChoose = LayoutInflater.from(this)
                .inflate(R.layout.item_toolbox_answer_options_abcd, null) as ViewGroup
            //第一次加载为选择题选项卡
            switchOptions(optionsChoose!!, false)
            sheetOptions = ArrayList()
        } else {
            logUtil("加载学生的答题选择器页面")
        }
    }

    private fun startAnswerTimerTeacher() {
        answerCountTimer = Observable.interval(1000, TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                if (progress > 0) {
                    progress--
                    teacherDialog.setTitle("结束倒计时:" + progress + "s")
                } else {
                    teacherDialog.dismiss()
                    answerCountTimer!!.dispose()
                    //由于两端本地都做了计时结束处理，因此当老师这边计时结束后
                    //1.请求历史答题数据
                    //2.本地存储答题数据更新
                    //TODO：这里选择1方法测试回调
                    liveRoom.toolBoxVM.requestAnswerPullReq(submitAnswer.id)
                }
            }
    }

    private fun startAnswerTimerStudent() {
        answerCountTimer = if (submitAnswer.isJudgement) {
            Observable.interval(1000, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    if (progress > 0) {
                        progress--
                        studentDialog.setTitle("回答倒计时：" + progress + "s")
                    } else {
                        studentDialog.dismiss()
                        //计时结束什么都不做
                        answerCountTimer!!.dispose()
                    }
                }
        } else {
            Observable.interval(1000, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    if (progress > 0) {
                        progress--
                        stuAlert.setTitle(
                            """
                                回答倒计时：${progress}s
                                ${submitAnswer.description}
                                """.trimIndent()
                        )
                    } else {
                        stuAlert.dismiss()
                        //什么都不做
                        answerCountTimer!!.dispose()
                    }
                }
        }
    }

    //切换选项布局
    private fun switchOptions(viewGroup: ViewGroup, isJudge: Boolean) {
        answerOptions!!.removeAllViews()
        answerOptions!!.addView(
            viewGroup, LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT
            )
        )

        //true为判断题
        submitQuestion!!.setType(isJudge)
    }

    fun onClickAnswer(view: View) {
        when (view.id) {
            R.id.answer_switch_abcd_bt -> switchOptions(optionsChoose!!, false)
            R.id.answer_switch_judge_bt -> switchOptions(optionsJudge!!, true)
            R.id.answer_submit_bt -> {
                val question = inputQuestion!!.text.toString()
                //答题时间30s
                submitQuestion!!.duration = 30
                //getDescription()做了空处理
                submitQuestion!!.description = question
                sheetOptions.clear()
                if (submitQuestion!!.isJudgement) {
                    //判断题
                    val radioTrue = optionsJudge!!.findViewById<RadioButton>(R.id.judge_true)
                    val raidoFalse = optionsJudge!!.findViewById<RadioButton>(R.id.judge_false)
                    if (!radioTrue.isChecked && !raidoFalse.isChecked) {
                        showToast("请选择一项作为判断题正确答案")
                        return
                    }
                    val lp1 = LPAnswerSheetOptionModel()
                    lp1.text = radioTrue.text.toString()
                    lp1.isRight = radioTrue.isChecked
                    sheetOptions.add(lp1)
                    val lp2 = LPAnswerSheetOptionModel()
                    lp2.text = raidoFalse.text.toString()
                    lp2.isRight = raidoFalse.isChecked
                    sheetOptions.add(lp2)
                } else {
                    //选择题
                    val ckA = optionsChoose!!.findViewById<CheckBox>(R.id.answer_ck_a)
                    val ckB = optionsChoose!!.findViewById<CheckBox>(R.id.answer_ck_b)
                    val ckC = optionsChoose!!.findViewById<CheckBox>(R.id.answer_ck_c)
                    val ckD = optionsChoose!!.findViewById<CheckBox>(R.id.answer_ck_d)
                    if (!ckA.isChecked && !ckB.isChecked &&
                        !ckC.isChecked && !ckD.isChecked
                    ) {
                        showToast("请至少选择一项作为选择题正确答案")
                        return
                    }
                    val lp1 = LPAnswerSheetOptionModel()
                    lp1.text = ckA.text.toString()
                    lp1.isRight = ckA.isChecked
                    sheetOptions.add(lp1)
                    val lp2 = LPAnswerSheetOptionModel()
                    lp2.text = ckB.text.toString()
                    lp2.isRight = ckB.isChecked
                    sheetOptions.add(lp2)
                    val lp3 = LPAnswerSheetOptionModel()
                    lp3.text = ckC.text.toString()
                    lp3.isRight = ckC.isChecked
                    sheetOptions.add(lp3)
                    val lp4 = LPAnswerSheetOptionModel()
                    lp4.text = ckD.text.toString()
                    lp4.isRight = ckD.isChecked
                    sheetOptions.add(lp4)
                }
                submitQuestion!!.options = sheetOptions
                //学生端回答需要使用
                submitQuestion!!.userId = liveRoom.currentUser.userId
                liveRoom.toolBoxVM.requestAnswerStart(submitQuestion)
            }
        }
    }

    private fun logUtil(mes: String) {
        Log.d(TAG, mes)
    }

}