package com.baijiayun.bjhl_liveplayer.activity.toolbox

import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager.widget.ViewPager
import com.baijiayun.bjhl_liveplayer.R
import com.baijiayun.bjhl_liveplayer.activity.BaseActivity
import com.baijiayun.bjhl_liveplayer.adapter.QaPagerAdapter
import com.baijiayun.bjhl_liveplayer.adapter.QaRecyclerCommonAdapter
import com.baijiayun.livecore.models.*
import com.google.android.material.tabs.TabLayout
import com.scwang.smartrefresh.layout.SmartRefreshLayout
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import io.reactivex.android.schedulers.AndroidSchedulers
import kotlinx.android.synthetic.main.activity_qa.*
import java.util.*

class QaActivity : BaseActivity() {
    private val TAG: String = "QaActivity"

    //待处理列表页数和已发布列表页数
    private var pendingPageCount = 0

    //上拉加载进度
    private var stepPendingCount = 0

    //每次下拉刷新都需要重置到stepPendingCount的计数器
    private var refreshPendingCount = 0

    //已发布
    private var publishPageCount = 0

    //上拉加载进度
    private var stepPublishCount = 0

    //每次下拉刷新都需要重置到stepPendingCount的计数器
    private var refreshPublishCount = 0
    private var questionAdapter: QaRecyclerCommonAdapter? = null
    private var publishAdapter: QaRecyclerCommonAdapter? = null
    private var questionEt: EditText? = null
    private var questionLists: MutableList<LPQuestionPullResItem>? = null
    private var publishLists: MutableList<LPQuestionPullResItem>? = null
    private lateinit var qaSmartLayoutPending: SmartRefreshLayout
    private lateinit var qaSmartLayoutPublish: SmartRefreshLayout
    // endregion

    override fun getContentViewId(): Int {
        return R.layout.activity_qa
    }

    override fun subscribe() {
        //提问回调
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfQuestionSendRes
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpQuestionSendModel: LPQuestionSendModel ->
                logUtil(
                    "id:" + lpQuestionSendModel.id
                            + ""
                            + " content:" + lpQuestionSendModel.content
                            + " status:" + lpQuestionSendModel.status
                            + " selfpagecount:" + lpQuestionSendModel.selfPageCount
                )
                pendingPageCount = lpQuestionSendModel.selfPageCount
                logUtil("提问回调，刷新获取提问列表")
            })

        //获取历史回答回调：包括发布、未发布、回复、未回复
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfQuestionPullRes
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpQuestionPullResModel: LPQuestionPullResModel ->
                when (lpQuestionPullResModel.status) {
                    8 -> {
                        logUtil("请求提问消息回调。" + "selfPageCount:" + lpQuestionPullResModel.selfPageCount)
                        pendingPageCount = lpQuestionPullResModel.selfPageCount
                        if (questionLists == null) {
                            questionLists = lpQuestionPullResModel.list
                        } else {
                            //上拉刷新下拉加载通用
                            questionLists!!.addAll(lpQuestionPullResModel.list)
                        }
                        //只有当上拉加载同步到下拉刷新的进度时，才能设置，每次下拉刷新都会同步两个计数器
                        if (refreshPendingCount >= stepPendingCount) {
                            qaSmartLayoutPending.finishRefresh()
                            qaSmartLayoutPending.finishLoadMore()
                            refreshPendingCount = stepPendingCount
                            questionAdapter!!.list = questionLists
                        }
                    }
                    1 -> {
                        publishPageCount = lpQuestionPullResModel.selfPageCount
                        if (publishLists == null) {
                            publishLists = lpQuestionPullResModel.list
                        } else {
                            //上拉刷新下拉加载通用
                            publishLists!!.addAll(lpQuestionPullResModel.list)
                        }
                        if (refreshPublishCount >= stepPublishCount) {
                            qaSmartLayoutPublish.finishRefresh()
                            qaSmartLayoutPublish.finishLoadMore()
                            refreshPublishCount = stepPublishCount
                            publishAdapter!!.list = publishLists
                        }
                    }
                }
            })

        //发布、回复相关回调
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfQuestionPub
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                logUtil("发布、回复相关回调，刷新获取发布列表")
            })

    }

    override fun initView() {
        initSchedule()
        val qaTab: TabLayout = qa_tab
        val qaPager: ViewPager = qa_pager
        val titleList: MutableList<String> = ArrayList()
        val pagerList: MutableList<View> = ArrayList()
        titleList.add("待处理列表")
        titleList.add("已发布列表")
        val pagerHandle =
            LayoutInflater.from(this).inflate(R.layout.item_toolbox_qa_pager_req, null) as ViewGroup
        val pagerPublish = LayoutInflater.from(this)
            .inflate(R.layout.item_toolbox_qa_pager_publish, null) as ViewGroup
        pagerList.add(pagerHandle)
        pagerList.add(pagerPublish)
        val qaPagerAdapter = QaPagerAdapter(titleList, pagerList)
        qaPager.adapter = qaPagerAdapter
        qaTab.setupWithViewPager(qaPager)
        initQuestionPagerView(pagerHandle)
        initPublishPagerView(pagerPublish)
        qaPager.offscreenPageLimit = 1
        requestPendingListFirst()
        requestPublishedListFirst()
    }

    //初始化问题Pager
    private fun initQuestionPagerView(pagerHandle: ViewGroup) {
        //question smart
        qaSmartLayoutPending = pagerHandle.findViewById(R.id.qa_smart_layout)
        qaSmartLayoutPending.setOnRefreshListener(OnRefreshListener { refreshLayout: RefreshLayout? ->
            //刷新前清空List
            questionLists?.clear()
            refreshPendingCount = 0
            while (refreshPendingCount <= stepPendingCount) {
                requestPendingList(refreshPendingCount)
                refreshPendingCount++
            }
        })
        qaSmartLayoutPending.setOnLoadMoreListener(OnLoadMoreListener { refreshLayout: RefreshLayout ->
            if (stepPendingCount <= pendingPageCount) {
                //加载下一页
                stepPendingCount++
                refreshPendingCount = stepPendingCount
                requestPendingList(stepPendingCount)
            } else {
                showToast("没有新的问题列表了")
                refreshLayout.finishLoadMore()
            }
        })

        //question Recycler
        val recyclerQuestion: RecyclerView = pagerHandle.findViewById(R.id.qa_common_recycler)
        questionAdapter = QaRecyclerCommonAdapter(this, R.layout.item_toolbox_qa_recyitem)
        questionAdapter!!.liveRoom = liveRoom
        //未回复Adapter
        questionAdapter!!.status = 8
        val qLM = LinearLayoutManager(this)
        recyclerQuestion.layoutManager = qLM
        recyclerQuestion.adapter = questionAdapter
        questionEt = pagerHandle.findViewById(R.id.qa_et)
    }

    //初始化发布Pager
    private fun initPublishPagerView(pagerPublish: ViewGroup) {
        //question smart
        qaSmartLayoutPublish = pagerPublish.findViewById(R.id.qa_smart_layout)
        qaSmartLayoutPublish.setOnRefreshListener(OnRefreshListener { refreshLayout: RefreshLayout? ->
            //刷新前清空List
            publishLists?.clear()
            refreshPublishCount = 0
            while (refreshPublishCount <= stepPublishCount) {
                requestPublishedList(refreshPublishCount)
                refreshPublishCount++
            }
        })
        qaSmartLayoutPublish.setOnLoadMoreListener(OnLoadMoreListener { refreshLayout: RefreshLayout ->
            if (stepPublishCount <= publishPageCount) {
                //加载下一页
                stepPublishCount++
                refreshPublishCount = stepPublishCount
                requestPublishedList(stepPublishCount)
            } else {
                showToast("没有新的发布列表了")
                refreshLayout.finishLoadMore()
            }
        })
        //publish Recycler
        val recyclerPublish: RecyclerView = pagerPublish.findViewById(R.id.qa_common_recycler)
        publishAdapter = QaRecyclerCommonAdapter(this, R.layout.item_toolbox_qa_recyitem)
        publishAdapter!!.liveRoom = liveRoom
        //已发布Adapter
        publishAdapter!!.status = 1
        val publishLM = LinearLayoutManager(this)
        recyclerPublish.layoutManager = publishLM
        recyclerPublish.adapter = publishAdapter
    }

    //设置请求历史问答的参数
    private fun setLPQuestionPullReqModel(
        status: Int,
        countPerPage: Int,
        isSelf: Int,
        pages: Int
    ): LPQuestionPullReqModel {
        val lpPullModel = LPQuestionPullReqModel()
        lpPullModel.status = status
        lpPullModel.countPerPage = countPerPage
        lpPullModel.isSelf = isSelf
        lpPullModel.page = pages
        return lpPullModel
    }

    //请求第一页待处理列表，只能初始化的时候调用
    private fun requestPendingListFirst() {
        requestPendingList(0)
    }

    //请求第page页的待处理列表
    private fun requestPendingList(page: Int) {
        val isSelf: Int = if (liveRoom.isTeacherOrAssistant
        ) {
            0
        } else {
            1
        }
        val lpPullModel = setLPQuestionPullReqModel(8, 10, isSelf, page)
        liveRoom.toolBoxVM.requestQuestionPullReq(lpPullModel)
    }

    //请求第一页已发布列表，同上
    private fun requestPublishedListFirst() {
        requestPublishedList(0)
    }

    //请求第page页的已发布列表
    private fun requestPublishedList(page: Int) {
        val lpPullModel = setLPQuestionPullReqModel(1, 10, 0, page)
        liveRoom.toolBoxVM.requestQuestionPullReq(lpPullModel)
    }

    //进入问答模块需要重置进度
    private fun initSchedule() {
        //上拉加载进度
        stepPendingCount = 0
        //每次下拉刷新都需要重置到stepPendingCount的计数器
        refreshPendingCount = 0
        //上拉加载进度
        stepPublishCount = 0
        //每次下拉刷新都需要重置到stepPendingCount的计数器
        refreshPublishCount = 0
    }

    private fun logUtil(mes: String) {
        Log.d(TAG, mes)
    }

    fun onClickQA(view: View) {
        when (view.id) {
            R.id.req_question_publish -> {
                val content = questionEt!!.text.toString()
                if ("" == content) {
                    showToast("请输入问题")
                    return
                }
                //发送问题请求
                liveRoom.toolBoxVM.requestQuestionSend(questionEt!!.text.toString())
            }
            R.id.clear_qa_et ->
                //清除输入框所有内容
                questionEt!!.setText("")
        }
    }
}