package com.baijiayun.bjhl_liveplayer.activity.toolbox

import android.util.Log
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import com.baijiayun.bjhl_liveplayer.R
import com.baijiayun.bjhl_liveplayer.activity.BaseActivity
import com.baijiayun.livecore.models.LPBJTimerModel
import com.baijiayun.livecore.models.LPDataUserModel
import com.baijiayun.livecore.models.LPTimerModel
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_timer.*
import java.util.concurrent.TimeUnit

class TimerActivity : BaseActivity() {
    private val TAG: String = "TimerActivity"
    private var inputTotal: EditText? = null
    private var timerSecond: TextView? = null
    private var timerMin: TextView? = null
    private var startTimer: Long = 0
    private var totalTime: Long = 0
    private var currentTime: Long = 0

    //默认正计时
    private var isCountDown = false
    private var localTimer: Disposable? = null
    private var isPause = false
    private var isOver = false

    override fun getContentViewId(): Int {
        return R.layout.activity_timer
    }

    override fun subscribe() {
        //开始计时响应
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfBJTimerStart
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpTimerModel: LPBJTimerModel ->
                startTimer = lpTimerModel.startTimer
                currentTime = lpTimerModel.current
                totalTime = lpTimerModel.total
                isCountDown = lpTimerModel.isCountDown
                startLocalTimer()
            })

        //暂停计时响应
        compositeDisposable.add(
            liveRoom.toolBoxVM.observableOfBJTimerPause
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { lpbjTimerModel: LPBJTimerModel ->
                    isPause = true
                    currentTime = lpbjTimerModel.current
                    totalTime = lpbjTimerModel.total
                    startTimer = lpbjTimerModel.startTimer
                    isCountDown = lpbjTimerModel.isCountDown
                    logUtil("计时暂停")
                })

        //停止计时响应
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfBJTimerEnd
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { aBoolean: Boolean? ->
                if (localTimer != null) {
                    localTimer!!.dispose()
                    localTimer = null
                }
                showToast("计时停止")
                totalTime = 0
                currentTime = 0
                setTimer(0)
                isPause = false
                isOver = false
            })

        //发起计时响应
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfTimerStart
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpTimerModel: LPTimerModel? -> })

        //撤回计时响应
        compositeDisposable.add(liveRoom.toolBoxVM.observableOfTimerRevoke
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lpDataUserModel: LPDataUserModel? -> })
    }

    override fun initView() {
        //加载计时器功能布局
        if (liveRoom.isTeacherOrAssistant) {
            timerSecond = timer_second
            timerMin = timer_min
            inputTotal = total_et
        } else {
            showToast("非老师不能使用计时器")
        }
    }

    private fun startLocalTimer() {
        if (localTimer == null) {
            logUtil("计时开始")
            localTimer = Observable.interval(100, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { aLong: Long? ->
                    if (!isPause && !isOver) {
                        logUtil("计时中")
                        setTimeOrTimeOver()
                    }
                }
            compositeDisposable.add(localTimer!!)
        }
    }

    private fun setTimeOrTimeOver() {
        if (totalTime < currentTime) {
            showToast("time error")
            setTimer(0)
        } else {
            if (isCountDown) {
                //倒计时
                if (currentTime <= 0) {
                    showToast("倒计时结束")
                    setTimer(0)
                    isOver = true
                } else {
                    setTimer(currentTime)
                    currentTime -= 1
                }
            } else {
                //正计时
                if (currentTime == totalTime) {
                    showToast("正计时结束")
                    setTimer(totalTime)
                    isOver = true
                } else {
                    setTimer(currentTime)
                    currentTime += 1
                }
            }
        }
    }

    private fun setTimer(currentTime: Long) {
        var miniuts: Long = 0
        var second: Long = 0
        if (currentTime >= 60) {
            miniuts = currentTime / 60
            second = currentTime % 60
        } else {
            second = currentTime
        }
        logUtil("设置Timer")
        timerSecond?.text = second.toString()
        timerMin?.text = miniuts.toString()
    }

    fun onClickTimer(view: View) {
        when (view.id) {
            R.id.req_timer -> showToast("1")
            R.id.revoke_timer -> showToast("2")
            R.id.start_time -> {
                //计时是否已经结束
                if (isOver) {
                    showToast("计时已经结束，请停止后再计时")
                }

                //没有调用dispose即停止计时，又按了开始按钮
                if (isPause) {
                    logUtil("继续计时")
                    isPause = false
                    return
                }
                if (localTimer == null) {
                    val total: String = inputTotal!!.text.toString()
                    if ("" == total) {
                        showToast("请输入最大值")
                        return
                    }
                    totalTime = total.toInt().toLong()
                    currentTime = if (isCountDown) totalTime else 0
                    liveRoom.toolBoxVM.requestBJTimerStart(
                        currentTime,
                        totalTime,
                        isCountDown
                    )
                }
            }
            R.id.pause_time -> liveRoom.toolBoxVM.requestBJTimerPause(currentTime, totalTime, false)
            R.id.stop_time -> if (localTimer != null) {
                liveRoom.toolBoxVM.requestBJTimerEnd()
            }
            R.id.forward_or_reverse -> {
                val button = view as Button
                if ("T" == button.text.toString()) {
                    //改成正计时
                    isCountDown = false
                    button.text = "F"
                } else {
                    //改成倒计时
                    isCountDown = true
                    button.text = "T"
                }
            }
        }
    }

    private fun logUtil(mes: String) {
        Log.d(TAG, mes)
    }

}