package com.baijiayun.bjhl_liveplayer.adapter;

import static com.baijiayun.bjhl_liveplayer.utils.Utils.convertTypeToString;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.BitmapDrawable;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.style.ImageSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.target.SimpleTarget;
import com.baijiayun.glide.request.transition.Transition;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.models.imodels.IMessageModel;

import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MessageAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    public static final int SEND_TEXT = 1;
    public static final int SEND_IMAGE = 2;
    public static final int RECEIVE_TEXT = 3;
    public static final int RECEIVE_IMAGE = 4;

    public final Pattern pattern = Pattern.compile("\\[[a-zA-Z0-9\u4e00-\u9fa5]+]");

    IUserModel currentUser;
    List<IMessageModel> datas;
    HashMap<String, String> expressions;
    Context mContext;
    private OnItemClickListener mOnItemClickListener;

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        mOnItemClickListener = onItemClickListener;
    }

    public MessageAdapter() {
    }

    public void setDatas(List<IMessageModel> datas) {
        this.datas = datas;
        notifyDataSetChanged();
    }

    public void addData(IMessageModel message) {
        datas.add(message);
        notifyDataSetChanged();
    }

    public void setExpressions(HashMap<String, String> expressions) {
        this.expressions = expressions;
    }

    public MessageAdapter(Context mContext, List<IMessageModel> datas, IUserModel currentUser) {
        this.datas = datas;
        this.mContext = mContext;
        this.currentUser = currentUser;
    }

    @Override
    public int getItemViewType(int position) {
        String fromUserNumber = datas.get(position).getFrom().getNumber();
        String messageType = datas.get(position).getMessageType().name();
        if (fromUserNumber.equals(currentUser.getNumber())) {
            if ("Image".equals(messageType)) {
                return SEND_IMAGE;
            } else {
                return SEND_TEXT;
            }
        } else {
            if ("Image".equals(messageType)) {
                return RECEIVE_IMAGE;
            } else {
                return RECEIVE_TEXT;
            }
        }
    }


    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (viewType == SEND_TEXT) {
            View view = LayoutInflater.from(mContext).inflate(R.layout.item_send_text, parent, false);
            return new SendTextViewHolder(view);
        } else if (viewType == RECEIVE_TEXT) {
            View view = LayoutInflater.from(mContext).inflate(R.layout.item_recieve_text, parent, false);
            return new ReceiveTextViewHolder(view);
        } else if (viewType == SEND_IMAGE) {
            View view = LayoutInflater.from(mContext).inflate(R.layout.item_send_image, parent, false);
            return new SendImageViewHolder(view);
        } else {
            View view = LayoutInflater.from(mContext).inflate(R.layout.item_receive_image, parent, false);
            return new ReceiveImageViewHolder(view);
        }
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        int type = getItemViewType(position);
        IMessageModel messageModel = datas.get(position);
        switch (type) {
            case SEND_TEXT:
                SendTextViewHolder sendTextViewHolder = (SendTextViewHolder) holder;
                sendTextViewHolder.tvMessageFrom.setText(messageModel.getFrom().getName());
                sendTextViewHolder.tvMessageContent.setText(getMixText(messageModel.getContent(), sendTextViewHolder.tvMessageContent));
                int sendMessageEndType = messageModel.getFrom().getEndType().getType();
                sendTextViewHolder.tvMessageEndType.setText(convertTypeToString(sendMessageEndType));
                Glide.with(mContext)
                        .load(messageModel.getFrom().getAvatar())
                        .into(sendTextViewHolder.ivAvatar);
                sendTextViewHolder.messageModel = messageModel;
                if (messageModel.isPrivateChat()) {
                    sendTextViewHolder.isPrivateChatTv.setVisibility(View.VISIBLE);
                    String privateChatInfo = String.format("私聊:%s", messageModel.getToUser().getName());
                    sendTextViewHolder.isPrivateChatTv.setText(privateChatInfo);
                }
                break;
            case RECEIVE_TEXT:
                ReceiveTextViewHolder receiveTextViewHolder = (ReceiveTextViewHolder) holder;
                receiveTextViewHolder.tvMessageFrom.setText(messageModel.getFrom().getName());
                receiveTextViewHolder.tvMessageContent.setText(getMixText(messageModel.getContent(), receiveTextViewHolder.tvMessageContent));
                int receiveMessageType = messageModel.getFrom().getEndType().getType();
                receiveTextViewHolder.tvMessageEndType.setText(convertTypeToString(receiveMessageType));
                Glide.with(mContext)
                        .load(messageModel.getFrom().getAvatar())
                        .into(receiveTextViewHolder.ivAvatar);
                receiveTextViewHolder.messageModel = messageModel;
                if (messageModel.isPrivateChat()) {
                    receiveTextViewHolder.isPrivateChatTv.setVisibility(View.VISIBLE);
                    String privateChatInfo = String.format("私聊:%s", messageModel.getFrom().getName());
                    receiveTextViewHolder.isPrivateChatTv.setText(privateChatInfo);
                }
                break;
            case SEND_IMAGE:
                SendImageViewHolder sendImageViewHolder = (SendImageViewHolder) holder;
                sendImageViewHolder.tvMessageFrom.setText(messageModel.getFrom().getName());
                int sendImageEndType = messageModel.getFrom().getEndType().getType();
                sendImageViewHolder.tvMessageEndType.setText(convertTypeToString(sendImageEndType));
                Glide.with(mContext)
                        .load(messageModel.getUrl())
                        .into(sendImageViewHolder.ivMessageContent);
                Glide.with(mContext)
                        .load(messageModel.getFrom().getAvatar())
                        .into(sendImageViewHolder.ivAvatar);
                if (messageModel.isPrivateChat()) {
                    sendImageViewHolder.isPrivateChatTv.setVisibility(View.VISIBLE);
                    String privateChatInfo = String.format("私聊:%s", messageModel.getToUser().getName());
                    sendImageViewHolder.isPrivateChatTv.setText(privateChatInfo);
                }
                sendImageViewHolder.messageModel = messageModel;
                break;
            case RECEIVE_IMAGE:
                ReceiveImageViewHolder receiveImageViewHolder = (ReceiveImageViewHolder) holder;
                receiveImageViewHolder.tvMessageFrom.setText(messageModel.getFrom().getName());
                int receiveImageEndType = messageModel.getFrom().getEndType().getType();
                receiveImageViewHolder.tvMessageEndType.setText(convertTypeToString(receiveImageEndType));
                String url = messageModel.getUrl();
                Glide.with(mContext)
                        .load(url)
                        .into(receiveImageViewHolder.ivMessageContent);
                Glide.with(mContext)
                        .load(messageModel.getFrom().getAvatar())
                        .into(receiveImageViewHolder.ivAvatar);
                if (messageModel.isPrivateChat()) {
                    receiveImageViewHolder.isPrivateChatTv.setVisibility(View.VISIBLE);
                    String privateChatInfo = String.format("私聊:%s", messageModel.getFrom().getName());
                    receiveImageViewHolder.isPrivateChatTv.setText(privateChatInfo);
                }
                receiveImageViewHolder.messageModel = messageModel;
                break;
            default:
                break;
        }
    }

    @Override
    public int getItemCount() {
        if (datas != null && datas.size() > 0) {
            return datas.size();
        }
        return 0;
    }

    public SpannableStringBuilder getMixText(String msgContent, TextView msgTv) {
        SpannableStringBuilder ssb = new SpannableStringBuilder(msgContent);
        Matcher matcher = pattern.matcher(msgContent);
        while (matcher.find()) {
            String group = matcher.group();
            if (expressions.containsKey(group)) {
                String url = expressions.get(group);
                URLDrawable bitmapDrawable = new URLDrawable();
                Glide.with(mContext).asBitmap().load(url).into(new SimpleTarget<Bitmap>() {
                    @Override
                    public void onResourceReady(@NonNull Bitmap bitmap, @Nullable Transition<? super Bitmap> transition) {
                        bitmapDrawable.bitmap = bitmap;
                        bitmapDrawable.setBounds(0, 0, bitmap.getWidth(), bitmap.getHeight());
                        msgTv.setText(msgTv.getText());
                    }
                });
                ImageSpan imageSpan = new ImageSpan(bitmapDrawable, ImageSpan.ALIGN_BASELINE);
                ssb.setSpan(imageSpan, matcher.start(), matcher.end(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
                ssb.removeSpan(group);
            }
        }
        return ssb;
    }

    public class ReceiveTextViewHolder extends RecyclerView.ViewHolder {
        private RelativeLayout rlReceiveText;
        private TextView tvMessageFrom;
        private TextView tvMessageContent;
        private TextView tvMessageEndType;
        private ImageView ivAvatar;
        private TextView isPrivateChatTv;
        private IMessageModel messageModel;

        public ReceiveTextViewHolder(View itemView) {
            super(itemView);
            rlReceiveText = itemView.findViewById(R.id.receive_text);
            tvMessageFrom = itemView.findViewById(R.id.message_from);
            tvMessageContent = itemView.findViewById(R.id.message_content);
            tvMessageEndType = itemView.findViewById(R.id.message_end_type);
            ivAvatar = itemView.findViewById(R.id.message_user_avatar);
            isPrivateChatTv = itemView.findViewById(R.id.is_private_chat_text);
            rlReceiveText.setOnLongClickListener(view -> {
                mOnItemClickListener.onItemLongClick(messageModel);
                return false;
            });
        }
    }

    public class SendTextViewHolder extends RecyclerView.ViewHolder {
        private RelativeLayout rlSendText;
        private TextView tvMessageFrom;
        private TextView tvMessageContent;
        private TextView tvMessageEndType;
        private ImageView ivAvatar;
        private TextView isPrivateChatTv;
        private IMessageModel messageModel;

        public SendTextViewHolder(View itemView) {
            super(itemView);
            rlSendText = itemView.findViewById(R.id.send_text);
            tvMessageFrom = itemView.findViewById(R.id.message_from);
            tvMessageContent = itemView.findViewById(R.id.message_content);
            tvMessageEndType = itemView.findViewById(R.id.message_end_type);
            ivAvatar = itemView.findViewById(R.id.message_user_avatar);
            isPrivateChatTv = itemView.findViewById(R.id.is_private_chat_text);
            rlSendText.setOnLongClickListener(view -> {
                mOnItemClickListener.onItemLongClick(messageModel);
                return false;
            });
        }
    }


    public class ReceiveImageViewHolder extends RecyclerView.ViewHolder {
        private RelativeLayout rlReceiveImage;
        private TextView tvMessageFrom;
        private ImageView ivMessageContent;
        private TextView tvMessageEndType;
        private ImageView ivAvatar;
        private TextView isPrivateChatTv;
        private IMessageModel messageModel;

        public ReceiveImageViewHolder(View itemView) {
            super(itemView);
            rlReceiveImage = itemView.findViewById(R.id.receive_image);
            tvMessageFrom = itemView.findViewById(R.id.message_from);
            ivMessageContent = itemView.findViewById(R.id.message_content);
            tvMessageEndType = itemView.findViewById(R.id.message_end_type);
            ivAvatar = itemView.findViewById(R.id.message_user_avatar);
            isPrivateChatTv = itemView.findViewById(R.id.is_private_chat_text);
            rlReceiveImage.setOnLongClickListener(view -> {
                mOnItemClickListener.onItemLongClick(messageModel);
                return false;
            });
        }
    }

    public class SendImageViewHolder extends RecyclerView.ViewHolder {
        private RelativeLayout rlSendImage;
        private TextView tvMessageFrom;
        private ImageView ivMessageContent;
        private TextView tvMessageEndType;
        private ImageView ivAvatar;
        private TextView isPrivateChatTv;
        private IMessageModel messageModel;

        public SendImageViewHolder(View itemView) {
            super(itemView);
            rlSendImage = itemView.findViewById(R.id.send_image);
            tvMessageFrom = itemView.findViewById(R.id.message_from);
            ivMessageContent = itemView.findViewById(R.id.message_content);
            tvMessageEndType = itemView.findViewById(R.id.message_end_type);
            ivAvatar = itemView.findViewById(R.id.message_user_avatar);
            isPrivateChatTv = itemView.findViewById(R.id.is_private_chat_text);
            rlSendImage.setOnLongClickListener(view -> {
                mOnItemClickListener.onItemLongClick(messageModel);
                return false;
            });
        }
    }

    public class URLDrawable extends BitmapDrawable {
        public Bitmap bitmap;

        @Override
        public void draw(Canvas canvas) {
            super.draw(canvas);
            if (bitmap != null) {
                canvas.drawBitmap(bitmap, 0, 0, getPaint());
            }
        }
    }

    public interface OnItemClickListener {
        void onItemLongClick(IMessageModel messageModel);
    }
}
