package com.baijiayun.bjhl_liveplayer.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjhl_liveplayer.R;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.context.LiveRoom;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

public class UserAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private List<UserCache> userList;
    private Context context;
    private LiveRoom liveRoom;
    private boolean isBlocked;
    /**
     * 维护一个待处理Number的队列，外界通过这个待处理队列来获取当前需要的Number，主要是用于无法获取Number的情况
     * param String -> Number
     */
    private Queue<String> pendingQueue;

    public UserAdapter() {

    }

    public UserAdapter(Context context, LiveRoom liveRoom, boolean isBlocked) {
        this.liveRoom = liveRoom;
        this.context = context;
        this.isBlocked = isBlocked;
        pendingQueue = new LinkedList<>();
    }

    public UserAdapter(Context context, LiveRoom liveRoom, List<UserCache> userList, boolean isBlocked) {
        this.context = context;
        this.userList = userList;
        this.isBlocked = isBlocked;
        this.liveRoom = liveRoom;
        pendingQueue = new LinkedList<>();
    }

    public void setUserlist(List<UserCache> userList) {
        this.userList = userList;
        notifyDataSetChanged();
    }

    public List<UserCache> getUserlist() {
        return userList;
    }

    public String pollFirstFromPending() {
        return pendingQueue.poll();
    }

    private void offerLastIntoPending(String number) {
        pendingQueue.offer(number);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.item_user, parent, false);
        return new ViewHolderUser(view);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        UserCache userCache = userList.get(position);
        IUserModel userModel = userCache.iUserModel;
        ViewHolderUser viewHolderUser = (ViewHolderUser) holder;
        viewHolderUser.userNameTv.setText(userModel.getName());
        viewHolderUser.userTypeTv.setText(userModel.getType().toString());

        TextView drawingAuthStatus = viewHolderUser.drawingStatusTv;
        switch (userCache.status.drawStatus) {
            case No:
                drawingAuthStatus.setText("未授权");
                break;
            case Have_DrawingAuth_Tea:
                // 保证只有老师才有的状态
                drawingAuthStatus.setText("已授予" + userModel.getName() + "画笔权限");
                break;
            case Have_DrawingAuth_Stu:
                // 学生被赋予画笔权限时的状态
                drawingAuthStatus.setText("已得到老师授予的画笔权限");
                break;
            default:
                break;
        }
        TextView speakStatus = viewHolderUser.speakStatusTv;
        switch (userCache.status.speakStatus) {
            case Online:
                speakStatus.setText("未发言");
                break;
            case HandsUp:
                speakStatus.setText(userModel.getName() + "请求举手");
                break;
            case HandsUp_Cancel:
                speakStatus.setText(userModel.getName() + "取消举手");
                break;
            case Invited:
                speakStatus.setText("邀请" + userModel.getName() + "发言");
                break;
            case Invited_Me:
                speakStatus.setText("老师邀请你发言");
                break;
            case Invited_Cancel:
                speakStatus.setText("取消了" + userModel.getName() + "发言邀请");
                break;
            case Invited_Cancel_Me:
                speakStatus.setText("老师取消了邀请");
                break;
            case Invited_Allow:
                speakStatus.setText(userModel.getName() + "同意邀请");
                // 这里其实应该有个被动触发的上麦动作，这个上麦动作伴随这音视频字段的开启，这里应该可以只是将状态改变为Speaking即可
                // 并且应该不需要将状态更新应用到源数据，因为此种状态都会到此分支来
                userCache.status.speakStatus = Status.SPEAK_STATUS.Speaking;
                // 文本与speaking设置不一致仅仅用于区别，实际上是一样的
                speakStatus.setText(userModel.getName() + "speaking(invite)");
                break;
            case Invited_Refuse:
                speakStatus.setText(userModel.getName() + "拒绝邀请");
                break;
            case Speaking:
                speakStatus.setText(userModel.getName() + "speaking");
                break;
            case Speaking_Stu_Cancel:
                speakStatus.setText(userModel.getName() + "主动下麦");
                break;
            case Speaking_Tea_Cancel:
                speakStatus.setText("老师关闭" + userModel.getName() + "的发言");
                break;
            default:
                break;
        }
        if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            // 隐藏学生speak布局
            viewHolderUser.speakOnlineStudent.setVisibility(View.GONE);

            // 黑名单
            Button setBlockedBt = viewHolderUser.blockedBts.findViewById(R.id.set_blocked_bt);
            Button cancelBlockedBt = viewHolderUser.blockedBts.findViewById(R.id.cancel_blocked_bt);

            if (isBlocked) {
                // 黑名单用户隐藏设置黑名单按钮
                setBlockedBt.setVisibility(View.GONE);
                // 隐藏画笔授权
                viewHolderUser.drawingAuth.setVisibility(View.GONE);
                // 隐藏发言
                viewHolderUser.speakOnlineTeacher.setVisibility(View.GONE);
                if (userModel.getType() == LPConstants.LPUserType.Teacher) {
                    cancelBlockedBt.setVisibility(View.GONE);
                } else {
                    cancelBlockedBt.setOnClickListener(v -> {
                        // 移除黑名单
                        liveRoom.getOnlineUserVM().freeBlockedUser(userModel.getNumber());
                    });
                }
            } else {
                // 在线用户隐藏移除黑名单按钮
                cancelBlockedBt.setVisibility(View.GONE);
                if (userModel.getType() == LPConstants.LPUserType.Teacher) {
                    viewHolderUser.blockedBts.setVisibility(View.GONE);
                    viewHolderUser.drawingAuth.setVisibility(View.GONE);
                    viewHolderUser.speakOnlineTeacher.setVisibility(View.GONE);
                } else {
                    setBlockedBt.setOnClickListener(v -> {
                        // 加入黑名单
                        liveRoom.getOnlineUserVM().requestKickOutUser(userModel.getUserId());
                    });

                    Button drawingAuthSetBt = viewHolderUser.drawingAuth.findViewById(R.id.drawing_auth_set_bt);
                    drawingAuthSetBt.setOnClickListener(v -> {
                        // 授权画笔
                        if (userCache.status.drawStatus == Status.DRAW_STATUS.No) {
                            offerLastIntoPending(userModel.getNumber());
                            liveRoom.getSpeakQueueVM().requestStudentDrawingAuthChange(true, userModel.getNumber());
                        }
                    });

                    Button drawingAuthCancelBt = viewHolderUser.drawingAuth.findViewById(R.id.drawing_auth_cancel_bt);
                    drawingAuthCancelBt.setOnClickListener(v -> {
                        // 取消授权画笔
                        if (userCache.status.drawStatus != Status.DRAW_STATUS.No) {
                            offerLastIntoPending(userModel.getNumber());
                            liveRoom.getSpeakQueueVM().requestStudentDrawingAuthChange(false, userModel.getNumber());
                        }
                    });

                    Button raiseHandAllowBt = viewHolderUser.speakOnlineTeacher.findViewById(R.id.raise_hand_allow_bt);
                    raiseHandAllowBt.setOnClickListener(v -> {
                        // 同意学生举手
                        liveRoom.getSpeakQueueVM().agreeSpeakApply(userModel.getUserId());
                        speakStatus.setText(String.format("老师同意%s举手", userModel.getName()));
                    });

                    Button raiseHandRefuseBt = viewHolderUser.speakOnlineTeacher.findViewById(R.id.raise_hand_refuse_bt);
                    raiseHandRefuseBt.setOnClickListener(v -> {
                        // 拒绝学生举手
                        liveRoom.getSpeakQueueVM().disagreeSpeakApply(userModel.getUserId());
                        speakStatus.setText(String.format("老师拒绝了%s举手请求", userModel.getName()));
                    });

                    Button exitSpeakBt = viewHolderUser.speakOnlineTeacher.findViewById(R.id.exit_speak_teacher_bt);
                    exitSpeakBt.setOnClickListener(v -> {
                        // 关闭学生发言
                        liveRoom.getSpeakQueueVM().closeOtherSpeak(userModel.getUserId());
                        speakStatus.setText(String.format("老师关闭了%s的发言", userModel.getName()));
                    });

                    Button inviteStudentBt = viewHolderUser.speakOnlineTeacher.findViewById(R.id.invite_student_bt);
                    inviteStudentBt.setOnClickListener(v -> {
                        // 邀请发言
                        liveRoom.getSpeakQueueVM().sendSpeakInviteReq(userModel.getUserId(), true);
                        speakStatus.setText("老师邀请中");
                    });

                    Button cancelInviteStudentBt = viewHolderUser.speakOnlineTeacher.findViewById(R.id.invite_student_cancel_bt);
                    cancelInviteStudentBt.setOnClickListener(v -> {
                        // 取消邀请发言
                        liveRoom.getSpeakQueueVM().sendSpeakInviteReq(userModel.getUserId(), false);
                        speakStatus.setText("老师取消了邀请");
                    });
                }
            }
        } else {
            //当前用户是学生
            //隐藏老师speak布局
            viewHolderUser.speakOnlineTeacher.setVisibility(View.GONE);
            viewHolderUser.blockedBts.setVisibility(View.GONE);
            viewHolderUser.drawingAuth.setVisibility(View.GONE);
            if (userModel.getUserId().equals(liveRoom.getCurrentUser().getUserId())) {
                Button raiseHandBt = viewHolderUser.speakOnlineStudent.findViewById(R.id.raise_hand_bt);
                raiseHandBt.setOnClickListener(v -> {
                    // 举手
                    liveRoom.getSpeakQueueVM().requestSpeakApply();
                    speakStatus.setText(String.format("%s正在请求举手", userModel.getName()));
                });

                Button cancelRaiseHandBt = viewHolderUser.speakOnlineStudent.findViewById(R.id.raise_hand_cancel_bt);
                cancelRaiseHandBt.setOnClickListener(v -> {
                    // 取消举手
                    liveRoom.getSpeakQueueVM().cancelSpeakApply();
                    speakStatus.setText(String.format("%s正在取消举手", userModel.getName()));
                });

                Button exitSpeakStuBt = viewHolderUser.speakOnlineStudent.findViewById(R.id.exit_speak_stu_bt);
                exitSpeakStuBt.setOnClickListener(v -> {
                    // 学生主动下麦
                    liveRoom.getSpeakQueueVM().cancelSpeakApply();
                    speakStatus.setText(String.format("%s主动下麦", userModel.getName()));
                });

                Button inviteAllowBt = viewHolderUser.speakOnlineStudent.findViewById(R.id.invite_student_allow_bt);
                inviteAllowBt.setOnClickListener(v -> {
                    // 同意邀请发言
                    liveRoom.getSpeakQueueVM().sendSpeakInvite(1);
                    speakStatus.setText("同意邀请并上麦发言");
                });

                Button inviteRefuseBt = viewHolderUser.speakOnlineStudent.findViewById(R.id.invite_student_refuse_bt);
                inviteRefuseBt.setOnClickListener(v -> {
                    // 拒绝邀请
                    liveRoom.getSpeakQueueVM().sendSpeakInvite(0);
                    speakStatus.setText("拒绝邀请");
                });
            } else {
                viewHolderUser.speakOnlineStudent.setVisibility(View.GONE);
            }
        }
    }

    @Override
    public int getItemCount() {
        return userList == null ? 0 : userList.size();
    }

    public static class ViewHolderUser extends RecyclerView.ViewHolder {
        public TextView userNameTv;
        public TextView userTypeTv;
        public TextView drawingStatusTv;
        public TextView speakStatusTv;
        public RelativeLayout blockedBts;
        public RelativeLayout drawingAuth;
        public RelativeLayout speakOnlineStudent;
        public RelativeLayout speakOnlineTeacher;

        public ViewHolderUser(@NonNull View itemView) {
            super(itemView);
            userNameTv = itemView.findViewById(R.id.userName);
            userTypeTv = itemView.findViewById(R.id.userType);
            drawingStatusTv = itemView.findViewById(R.id.drawing_auth_status);
            speakStatusTv = itemView.findViewById(R.id.speak_status);
            blockedBts = itemView.findViewById(R.id.blocked_bts);
            drawingAuth = itemView.findViewById(R.id.drawing_auth);
            speakOnlineTeacher = itemView.findViewById(R.id.speak_online_teacher);
            speakOnlineStudent = itemView.findViewById(R.id.speak_online_student);
        }
    }

    public static class UserCache {
        public IUserModel iUserModel;

        public Status status = new Status();
    }

    public static class Status {
        public SPEAK_STATUS speakStatus = SPEAK_STATUS.Online;

        public DRAW_STATUS drawStatus = DRAW_STATUS.No;

        public enum SPEAK_STATUS {
            // 可能状态需要设置优先级
            Online,//default
            Invited,
            Invited_Me,
            Invited_Cancel,
            Invited_Cancel_Me,
            Invited_Allow,
            Invited_Refuse,
            HandsUp,
            HandsUp_Cancel,
            Speaking,
            Speaking_Tea_Cancel,
            Speaking_Stu_Cancel,
            Blocked,
        }

        public enum DRAW_STATUS {
            No,
            Have_DrawingAuth_Stu,
            Have_DrawingAuth_Tea,
            Blocked,
        }
    }
}
